/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

	// declare the Dojo module
	dojo.provide("carter.ui.PublishWidget");

	//declare dependencies on other Dojo modules
	dojo.require("jazz.app.context");

	//declare dependencies on other Dojo modules
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

	dojo.require("dijit.form.Button");
	dojo.require("dijit.form.DateTextBox");
	dojo.require("dijit.form.TimeTextBox");
	dojo.require("dijit.form.TextBox");
	dojo.require("dijit.form.FilteringSelect");

	dojo.require("jazz.client.xhr");
	dojo.require("jazz.app.auth");

	dojo.require("dojo._base.xhr");

	dojo.require("jazz.ui.Dialog");
	dojo.require("jazz.ui.FilterBox");

	dojo.require("net.jazz.ajax.ui.PlatformUI");

	dojo.require("jazz.ui.SideNavigationLayout");

    dojo.require("rmps.app.ui.EditorUtil");
    dojo.require("rmps.app.ui.EditorView");

	dojo.require("carter.ui.utils");
	dojo.require("carter.ui.PublishWidgetController");
	dojo.require("carter.ui.publisher.NewPublishEditorInput");
	dojo.require("carter.ui.publisher.ManageModelsEditorInput");
	dojo.require("carter.ui.editors.DefinitionsEditorInput");
	dojo.require("carter.ui.editors.DefinitionEditorInput");
	dojo.require("carter.ui.editors.ActivitiesEditorInput");
		
	dojo.require("dojo.i18n");
	dojo.requireLocalization("carter.ui", "PublisherMessages");
	var messages = dojo.i18n.getLocalization("carter.ui", "PublisherMessages");

	var Utils = carter.ui.utils;	
	var PAGE_ID = "carter.ui.publish";
	var ACTION_PREFIX = "carter.ui.publish.";
	var ACTION_NAMES = [
		"_default",
		"viewNewPublish",
		"viewManageModels",
		"viewDefinitions",
		"viewDefinitionEditor",
		"viewActivities"
	];

dojo.declare("carter.ui.PublishWidget", [ dijit._Widget ], {
	
	"class": "carter-ui-PublishWidget",

	// private properties
	_sideNav: null,
	_controller: null,
	_PublishViewInput: null,
	_ManageView: null,
	_DefinitionsView: null,
	_DefinitionEditorInput: null,
	_ActivitiesEditorInput: null,
	
	_NavOptionsBtn: null,
	_PublishNavSection: null,
	_ManageNavSection: null,
	_DefinitionsSection: null,
	_ActivitiesSection: null,
	
	_deferred: null,
	
	postCreate: function() {
				
        // create editor util instance
        this._editorUtil = rmps.app.ui.EditorUtil.getInstance(PAGE_ID, this.domNode);
		
		// create controller        
        this._controller = new carter.ui.PublishWidgetController(this);
				
		// register actions
        var actionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
		for (var i = 0; i < ACTION_NAMES.length; i++) {
			actionRegistry.registerAction(ACTION_PREFIX + ACTION_NAMES[i], this._controller, ACTION_NAMES[i]);
		}
		
        // Create editor inputs:
        var projectId = jazz.app.context.get().itemId;
        this._PublishViewInput = new carter.ui.publisher.NewPublishEditorInput({ projectId: projectId });
		this._ManageView = new carter.ui.publisher.ManageModelsEditorInput({ projectId: projectId });
		this._DefinitionsView = new carter.ui.editors.DefinitionsEditorInput({ projectId: projectId });
		this._DefinitionEditorInput = new carter.ui.editors.DefinitionEditorInput({ projectId: projectId });
		this._ActivitiesEditorInput = new carter.ui.editors.ActivitiesEditorInput({ projectId: projectId });
		                
		// initialize the side navigation
		this._sideNav = new jazz.ui.SideNavigationLayout();
		this.domNode.appendChild(this._sideNav.domNode);
		this._sideNav.startup();
				
		var section = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), messages.rhapsodyImport, true);
		var hiddenSection = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), "Hidden section", true);
		if( hiddenSection != null ){
			hiddenSection.setVisible(false);
			hiddenSection.addSectionRow("Definition Editor", Utils.getDefinitionEditorUri(), null, null);
		}
		
        this._DefinitionsSection = section.addSectionRow(
        		this._DefinitionsView.getName(),
				Utils.getDefinitionsViewUri(),
				this._DefinitionsView.getIconUri(), null);

        this._PublishNavSection = section.addSectionRow(
        		this._PublishViewInput.getName(),
				Utils.getNewPublishViewUri(),
				this._PublishViewInput.getIconUri(), null);
						
		this._ManageNavSection = section.addSectionRow(
				this._ManageView.getName(),
				Utils.getManageModelsViewUri(),
				this._ManageView.getIconUri(), null);
                
		this._ActivitiesSection = section.addSectionRow(
				this._ActivitiesEditorInput.getName(),
				Utils.getActivitiesEditorUri(),
				this._ActivitiesEditorInput.getIconUri(), null);

        var ThePublishView = this._getPublishView();
        this._sideNav.setMainContent(ThePublishView.domNode);
	},
	
    _getPublishView: function() {
        if (!this._NewPublishView) {
            this._NewPublishView = new rmps.app.ui.EditorView({
                editorUtil: this._editorUtil
            });
          
            this._NewPublishView.startup();
            this._editorUtil.domNode = this._NewPublishView.domNode;
            this._editorUtil.defaultUri = Utils.getNewPublishViewUri();
        }        
        return this._NewPublishView;
    },

	showNewPublishView: function(args) {
		this._highlightNode(this._PublishNavSection);
		this._editorUtil.openEditor(this._PublishViewInput);		
	},
	
	showManageModelsView: function(args) {
		this._highlightNode(this._ManageNavSection);
		this._editorUtil.openEditor(this._ManageView);
	},
	
	showDefinitionsView: function(args) {
		this._highlightNode(this._DefinitionsSection);
		this._editorUtil.openEditor(this._DefinitionsView);
		
		if( args.param1 == "defSavedOk" ) {
			var defListEditor = this._editorUtil.getEditor(this._DefinitionsView);
			var defEditor = this._editorUtil.getEditor(this._DefinitionEditorInput);
			if( defListEditor && defEditor ) {
				var def = defEditor.getSavedDefinition();
				defListEditor.addDefinition(def);
			}
		}
	},

	showDefinitionEditor: function(args) {
		this._highlightNode(this._DefinitionsSection);
		if( this._DefinitionEditorInput.isLoaded()){
			this._editorUtil.openEditor(this._DefinitionEditorInput);
			var defEditor = this._editorUtil.getEditor(this._DefinitionEditorInput);
			if( defEditor != null ) {
				defEditor.loadContent(args.itemId);
			}			
		}						
		else {
			this._loadDefinitionEditor(args.itemId);
			this._editorUtil.openEditor(this._DefinitionEditorInput);
		}
	},
	
	showActivitiesEditor: function(args) {
		this._highlightNode(this._ActivitiesSection);
		this._editorUtil.openEditor(this._ActivitiesEditorInput);
		var editor = this._editorUtil.getEditor(this._ActivitiesEditorInput);
		if( editor ){
			editor.refresh();
		}
	},

	_highlightNode: function(node){
		if (this._highlightedNode != node) {
			if (this._highlightedNode) {
				dojo.removeClass(this._highlightedNode, "selected");
			}
			dojo.addClass(node, "selected");
			this._highlightedNode = node;
		}
	},
	
	_loadDefinitionEditor: function(defId){
		if( this._DefinitionEditorInput == null || this._DefinitionEditorInput.isLoaded() ) return;
		
		this._editorUtil._cacheEditor(this._DefinitionEditorInput);
		
		this._deferred = this._DefinitionEditorInput.load();
		
		if( this._deferred != null ) {
			this._deferred.addCallback(this, function() {
				delete this._deferred;
				this._editorUtil._openEditor(this._DefinitionEditorInput);
				var defEditor = this._editorUtil.getEditor(this._DefinitionEditorInput); 
				if( defEditor != null ){
					defEditor.loadContent(defId);
				}				
			});
		
			this._deferred.addErrback(this, function(error) {
				delete this._deferred;
			});
		}
	}
});

})();

