/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

	// declare this Dojo module
	dojo.provide("carter.ui.admin.AdminWidget");

	// declare other modules:
	dojo.require("rmps.core.RootServices");
	dojo.require("jazz.client.xhr");
	dojo.require("dijit._Widget");
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	dojo.require("jazz.util.ResizeNotifier");
	dojo.require("dojox.xml.DomParser");

	dojo.require("jazz.ui.SideNavigationLayout");
	dojo.require("rmps.app.ui.EditorUtil");
    dojo.require("rmps.app.ui.EditorView");
			
	dojo.require("carter.ui.RootServices");
	dojo.require("carter.ui.publisher.NewPublishEditorInput");		
	dojo.require("carter.ui.utils");
	dojo.require("carter.ui.admin.AdminController");
	
	var Utils = carter.ui.utils;
	var PAGE_ID = "carter.ui.admin";	
	var ACTION_PREFIX = "carter.ui.admin.";
	var DEF_ACTION = "_showConfigurationPage";

	var ACTION_NAMES = [
	            		"viewAdminConfig"
	            	];

	dojo.declare("carter.ui.admin.AdminWidget", [ dijit._Widget ], {
	
	"class": "carter-ui-admin-AdminWidget",

	// private properties
	_sideNav: null,
	_RhpConfig_NavSection: null,
	_controller: null,
	_ConfigurationView: null,
	_ConfigurationEditorInput: null,
	
	pageId: "carter.ui.admin",
	uri: rmps.core.RootServices.getUri(carter.ui.Namespaces.CARTER, "configuration"),
	
	postCreate: function() {
		
		// create editor util instance
        this._editorUtil = rmps.app.ui.EditorUtil.getInstance(PAGE_ID, this.domNode);
        
		// create controller    
        this._controller = new carter.ui.admin.AdminController(this);
        
		// register actions
        var actionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
		for (var i = 0; i < ACTION_NAMES.length; i++) {
			actionRegistry.registerAction(ACTION_PREFIX + ACTION_NAMES[i], this._controller, ACTION_NAMES[i]);
		}
		
		this._resizeHandle = jazz.util.ResizeNotifier.register({context: this, callback: this._handleWindowResize, force: true});
					
		// initialize the side navigation
		//this._sideNav = new jazz.ui.SideNavigationLayout();
		//this.domNode.appendChild(this._sideNav.domNode);
		//this._sideNav.startup();
				
		//var section = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), "Rhapsody Configuration", true);
				
		//this._RhpConfig_NavSection = section.addSectionRow(
        //"Model Server Configuration",
        //this.uri,
		//null, null);
	},
	
    _InitConfigurationView: function() {
        if (!this._ConfigurationView) {
            this._ConfigurationView = new rmps.app.ui.EditorView({
                editorUtil: this._editorUtil
            });
          
            this._ConfigurationView.startup();
            this._editorUtil.domNode = this._ConfigurationView.domNode;
            this._editorUtil.defaultUri = Utils.getConfigurationEditorUri();
        }        
        return this._ConfigurationView;
    },
	
	destroy: function() {
		jazz.util.ResizeNotifier.unregister(this._resizeHandle);
		this.inherited(arguments);
	},
	
	_handleWindowResize: function(event) {
		setTimeout(dojo.hitch(this, this.resize), 0);
	},
	
	resize: function() {
		if (net.jazz.ajax.ui.PlatformUI.getWorkbench().currentPageId === this.pageId) {
			var contentCell = dojo.byId("content-cell");
			var c1 = dojo.coords(contentCell, true);
			var c2 = dojo.coords(this.domNode, true);
			var height = (parseInt(contentCell.style.height) || parseInt(contentCell.style.minHeight)) + c1.y - c2.y;
			this.domNode.style.height = height + "px";
		}
	},
	
	_showConfigurationPage: function() {
		jazz.client.xhrGet({
			url: this.uri,
			load: dojo.hitch(this, function(response) {
				var doc = dojox.xml.DomParser.parse(response);
				var url = this.uri;
				dojo.create("iframe", {style: "display:block; width:100%; border:none; height:100%;", src: url, frameborder: "0", height: "100%", width: "100%"}, this.domNode);
			})
		});
	}
});

})();
