/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2010.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/
// declare the Dojo module
dojo.provide("carter.ui.editors.ActivitiesEditor");

// Dojo requirements
dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.io.iframe");

//jazz/kenton requirements
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.context");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.StyledTable");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.FilterBox");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("jazz.util.date");

// Carter reuqirements:
dojo.requireLocalization("carter.ui", "EditorMessages");
dojo.require("carter.ui.utils");
dojo.require("carter.ui.editors.ActivitiesMgr");
dojo.require("carter.ui.editors.Activity");
dojo.require("carter.ui.error");

// globals:
var messages = dojo.i18n.getLocalization("carter.ui", "EditorMessages");
var Utils = carter.ui.utils;

var ACTTYPE_MODELIMPORT = messages.modelimport;
var ACTTYPE_MODELREMOVE = messages.modelremoved;
var ACTTYPE_DEFIMPORT 	= messages.importByEngine;
var ACTTYPE_UNKNOWN 	= messages.unknown;

var STATE_ENDED		= 1;
var STATE_NOTENDED	= 2;

var ITEMS_LIMIT		= 25;

// Context help IDs:
var UAKEY_ACTIVITIES_PAGE = "com.ibm.rational.carter.web.page_ActivitiesEditor01";

// Editor implementation:
dojo.declare("carter.ui.editors.ActivitiesEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "carter-ui-editors-activitiesEditor",
	actList: null,
	
	constructor: function(arguments){
		// set context help id to the editor
		this.uaKey = UAKEY_ACTIVITIES_PAGE;		
	},

    postCreate: function() {
        this.inherited(arguments);
        
        this._init();
    },
     
	addButtons: function(toolbar) {		
		this.inherited(arguments);

		// hide editor buttons: 
		this.saveButton.domNode.style.display = "none";
		
		toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.showAll,
			onClick: dojo.hitch(this, this.onShowAll)
		}));		
		
		toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.deleteAll,
			onClick: dojo.hitch(this, this.onDeleteAll)
		}));		

	},
	
	_init: function() {
        this.actList = new carter.ui.editors.ActivitiesEditor._actList({editor: this});
        this.containerNode.appendChild(this.actList.domNode);
        this.actList.startup();		
    },
    
    refresh: function() {
		if( this.actList ) {
			this.actList.loadContent(ITEMS_LIMIT);
		}
    },
    
    onShowAll: function() {
		if( this.actList ) {
			this.actList.loadContent(0);
		}    	
    },
    
    onDeleteAll: function() {
		if( this.actList ) {
			this.actList.clearContent();
		}    	
    },
    
	showInfoMsg: function(msg){
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: msg,
			closable: true
		});
	},
	
	showOkMsg: function(msg){
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: msg,
			closable: true
		});
	},

	showErrorMsg: function(msg){
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: msg,
			closable: true
		});
	},

	hideMsg: function(){
		this.hideMessage();
	}
});

// Widget implementation:
dojo.declare("carter.ui.editors.ActivitiesEditor._actList", [dijit._Widget, dijit._Templated], {
	
	messages: null,
	widgetsInTemplate: true,
	editor: null,
	_updateTimer: null,
	_deferred: null,
	_clearDeferred: null,
	
	templateString: '<div>' +
						'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="false""></div>' +
					'</div>',

	constructor: function(editor){
		this.editor =  editor;
		this.messages = messages;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		 
		//create table
		this._table = new jazz.ui.StyledTable({
			columns: [
						{label: "", width: "50px"},
						{label: this.messages.type, width: "15%"},
						{label: this.messages.details, width: "20%"},
						{label: this.messages.requester, width: "20%"},
						{label: this.messages.requestTime, width: "15%"},
						{label: this.messages.startTime, width: "15%"},
						{label: this.messages.duration, width: "15%"},
						{label: this.messages.actStatus, width: "50%"},
						{label: this.messages.actions, className: "actions", width: "10%"}
					]});
	
		this._section.headerNode.style.display = "none";
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
		
		this.loadContent(ITEMS_LIMIT);
		
		// start update interval: 
		if( ! this._updateTimer ){
			this._updateTimer = setInterval(dojo.hitch(this, this.loadContent), Utils.TIMEINTERVAL_ACTIVITIES_REFRESH);
		} 
	},
	
    destroy: function() {
   		if (this._updateTimer) {
			clearInterval(this._updateTimer);
			delete this._updateTimer;
		} 	
		if (this._deferred) {
			this._deferred.cancel();
			delete this._deferred;
		}
        this.inherited(arguments);
    },
	
	loadContent: function(itemsLimit) {
		
		// prevent requests overlapping: 
		if (this._deferred) {
			return;
		}
		
		if( this.editor != null && this.messages != null )
			this.editor.showInfoMsg(this.messages.loading);
			
		// Clear previous content...
		this.resetContent();
		
		// query for data:
		var actMgr = carter.ui.editors.ActivitiesMgr;
		if( ! actMgr ){
			alert(messages.err_NoActivitiesManger);
			return;
		}

		this._deferred = actMgr.getActivities(itemsLimit);
		
		// Populate the table:
		if( this._deferred != null ){
			this._deferred.addCallback(this, function(activities) {
				
				if( activities == null ) return;
				
				function activityComparator(a, b) {
					return a.requestTime < b.requestTime ? 1 : a.requestTime == b.requestTime ? 0 : -1;
				}
				
				activities.sort(activityComparator);
				
				for( var i = 0 ; i < activities.length; ++i) {
					this.addActivity(activities[i]);
				}
			});			
			this._deferred.addErrback(this, function(error) {

				var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}

				if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
				} 
				else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			this._deferred.addBoth(this, function() {
				delete this._deferred;
			});			
		}
		
		if( this.editor != null )
			this.editor.hideMsg();
	},
	
	clearContent: function() {
		if( this._clearDeferred != null ) return false;
		
		if( this.count() < 1 ) return false;
		
		var actMgr = carter.ui.editors.ActivitiesMgr;
		if( ! actMgr ){
			alert(messages.err_NoActivitiesManger);
			return false;
		}
				
		var ret = false;
		this._clearDeferred = actMgr.deleteActivity("all");
			
		if( this._clearDeferred != null ){			
			this._clearDeferred.addCallback(this, function(result) {
				ret = result;
				if( this.editor != null )
				{
					this.editor.showOkMsg(this.messages.actDeleteAllOk);
				}
				this.removeEndedContent();
			});
			this._clearDeferred.addErrback(this, function(error) {
				var errMsg = carter.ui.error.getCarterError(this._clearDeferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._clearDeferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._clearDeferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});	
			this._clearDeferred.addBoth(this, function() {
				delete this._clearDeferred;
			});		
		}

		return ret;
    },
	
	addActivity: function(activity) {
		var row = new carter.ui.editors.Activities._ActivityRow({list: this, actObj: activity});
		this._table.tbody.appendChild(row.domNode);
	},
	
	addOrUpdateActivity: function(activity){
/*
		var row = this.findActivityRow(defId)
		if( row != null ) {
			row.setName(defName);
		}
		else {
			this.addDefToList(defName, defId);
		}
*/
	},
	
	findActivityRow: function(actId){
		var retRow = null;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			if( row.getId() == actId ) {
				retRow = row;
			}
		});	
		return retRow;		
	},
	
	resetContent: function() {
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			row.destroyRecursive();
			delete row;
		});
		
		dojox.xml.parser.removeChildren(this._table.tbody);
	},
	
	removeEndedContent: function() {
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			if( row.isEnded() )
			{
				row.destroyRecursive();
				delete row;
			}
		});
	},

	count: function() {
		var nCount = 0;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			nCount++;
		});
		return nCount;
	}
});

dojo.declare("carter.ui.editors.Activities._ActivityRow", [dijit._Widget, dijit._Templated], {

    widgetsInTemplate: true,
    _deleteDeferred: null,
    _cancleDeferred: null,
    
    templateString: '<tr>' +
						'<td>' +
							'<div>' +
								'<img dojoAttachPoint="_stateIcon"></img>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_actType"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_details"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_requester"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_requestTime"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_startTime"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_duration"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_status"></span>' +
							'</div>' +
						'</td>' +
						'<td class="actions"">' +
							'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
						'</td>' +
					'</tr>',

    messages: null,
    list: null,
	editor: null,
	_activity: null,
    _cancelDeferred: null,
    _popupContent: null,
    _popup: null,
    _actionBar: null,
	_updateTimer: null,
	_updateDeferred: null,
	_originalState: null,
	_btnCancel: null,
	_btnDelete: null,

    constructor: function(arguments) {
        this.list = arguments.list;
        if( this.list ) this.editor = this.list.editor;
        this.messages = messages;
        
        if( arguments.actObj){
        	this._activity = dojo.clone(arguments.actObj);	
        }
    },
    postCreate: function() {
    	if( this._activity != null )
    	{
        	this.updateContent(this._activity, false);
    	}
    },
    
	updateContent: function(activity, doClone){
		if( activity ){
			if( doClone ){
				if( this._activity != null )
					delete this._activity;
				this._activity = dojo.clone(activity); 
			}
			
			if( this._activity == null ) return;
			
			this.setType(this._activity.activityType);
			this.setRequester(this._activity.requester);
			this.setRequestTime(this._activity.getRequestTimeStr());
			this.setStartTime(this._activity.getStartTimeStr());
			this.setDuration();
			this.setState(this._activity.state);
			
			if( this._activity.details!=null ) {
				this.setDetails(this._activity.details);
			}
			
			// Keep original state:
			if( this._originalState == null )
			{
				if( this._activity.isEnded() == false )
					this._originalState = STATE_NOTENDED;
				else
					this._originalState = STATE_ENDED;
			}
			
			// Update action buttons:
			if( this._activity.isEnded()){
				if( this._btnCancel != null ){
					this.toolbar.removeItem(this._btnCancel);
					delete this._btnCancel;
				}
				if( this._btnDelete == null ){
        			this._btnDelete = this.toolbar.addButton(this, function(e) {
            			dojo.stopEvent(e);
            			if( this._deleteActivity() ) {
            			}
        			}, {
            			label: this.messages.msgDelete,
            			imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/remove.gif")
        			});        			
				}
			}
			else {
				if( this._btnDelete != null ){
					this.toolbar.removeItem(this._btnDelete);
					delete this._btnDelete;
				}				
				if( this._btnCancel == null ){
        			this._btnCancel = this.toolbar.addButton(this, function(e) {
            							dojo.stopEvent(e);
            							if( this._cancelActivity() ) {
            							}
        								}, {
            								label: this.messages.msgCancel,
            								imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/cancel.gif")
        								});
				}
			}
			
			// start update interval if needed or disable the cancel button: 
			if( this._activity.isEnded() == false ){
				this._updateTimer = setInterval(dojo.hitch(this, this.updateState), Utils.TIMEINTERVAL_ACTIVITY_PROGRESS);
				if( this._btnCancel != null ){
					this.toolbar.disableButton(this._btnCancel);
				}
			}
			else {
				if( this._originalState && this._originalState == STATE_NOTENDED )
				{
					dojo.publish("rmps.web.app.explorer.refresh");
				}
			}			
		}	
	},
	
	isEnded: function(){
		if( this._activity )
			return this._activity.isEnded();
		return false;
	},
	
    setType: function(type) {
    	if(type == null ) return;
        this._actType.innerHTML = "";
        var displayType = this._getDisplayType(type);
        if( displayType == null )
        	displayType = type;
        this._actType.appendChild(dojo.doc.createTextNode(displayType));
    },
    
    setRequester: function(requester) {
    	if(requester == null ) return;
        this._requester.innerHTML = "";    
		this._requester.appendChild(dojo.doc.createTextNode(requester));
    },
    
    setRequestTime: function(requestTime) {
    	if(requestTime == null ) return;
        this._requestTime.innerHTML = "";
        this._requestTime.appendChild(dojo.doc.createTextNode(requestTime)); 
    },	
    setStartTime: function(startTime) {
    	if(startTime == null ) return;
        this._startTime.innerHTML = "";
        this._startTime.appendChild(dojo.doc.createTextNode(startTime)); 
    },	
    setEndTime: function(endTime) {
    	if(endTime == null ) return;
        this._endTime.innerHTML = "";
        this._endTime.appendChild(dojo.doc.createTextNode(endTime)); 
    },	
    setState: function(state) {
    	if(state == null ) return;
    	this._stateIcon.src = state.icon;
        this._status.innerHTML = "";
        this._status.appendChild(dojo.doc.createTextNode(state.name)); 
    },	
    setDetails: function(details) {
    	if(details == null ) return;
        this._details.innerHTML = "";
        this._details.appendChild(dojo.doc.createTextNode(details)); 
    },	
    
	getId: function() {
		if( this._activity != null ){
			return this._activity.actId;
		}
		return 0;
	},
    destroy: function() {
   		if (this._updateTimer) {
			clearInterval(this._updateTimer);
			delete this._updateTimer;
		} 	
		if (this._updateDeferred) {
			this._updateDeferred.cancel();
			delete this._updateDeferred;
		}
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
            delete this._popupContent;
        }
        if (this._popup) {
            this._popup.destroyRecursive();
            delete this._popup;
        }
        this.inherited(arguments);
    },
    
    _cancelActivity: function() {
    	alert("cancel activity request");
    	return this.cancelActivity();
    },
    
    _deleteActivity: function() {
		if( this._activity == null ) return false;
		if( this._deleteDeferred != null ) return false;
		
		var actMgr = carter.ui.editors.ActivitiesMgr;
		if( ! actMgr ){
			alert(messages.err_NoActivitiesManger);
			return false;
		}
				
		var ret = false;
		this._deleteDeferred = actMgr.deleteActivity(this._activity.requestId);
			
		if( this._deleteDeferred != null ){			
			this._deleteDeferred.addCallback(this, function(result) {
				ret = result;
				if( this.editor != null )
				{
					this.editor.showOkMsg(this.messages.actDeleteOk);
				}
				this.destroyRecursive();
				delete this;
			});
			this._deleteDeferred.addErrback(this, function(error) {
				var errMsg = carter.ui.error.getCarterError(this._deleteDeferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._deleteDeferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deleteDeferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});	
			this._deleteDeferred.addBoth(this, function() {
				delete this._deleteDeferred;
			});		
		}

		return ret;
    },
    _createPopupContents: function() {
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
        }
    },
    
    setDuration: function() {
    	if( this._activity == null ) return;
		this._duration.innerHTML = "";
		
		var tStart = this._activity.startTime;
		var tEnd = this._activity.stopTime;
					
		//if the request was canceled or dropped before it could started then use request time for calculation 
		if (tStart==null && tEnd!=null && this._activity.isEnded()) {
			tStart = this._activity.requestTime;
		} 

		if (tStart != null && tEnd != null ) {
			var timeSpan = jazz.util.date.formatDuration(tEnd.getTime() - tStart.getTime());
			this._duration.appendChild(dojo.doc.createTextNode(timeSpan));
		} else {
			this._duration.appendChild(dojo.doc.createTextNode(" - "));
		}
	},
    
    _getDisplayType: function(type){    	
  		if( type == Utils.ACTIVITY_TYPE_MODELIMPORT ) 		return ACTTYPE_MODELIMPORT;
		if( type == Utils.ACTIVITY_TYPE_MODELDELETION ) 	return ACTTYPE_MODELREMOVE;
		if( type == Utils.ACTIVITY_TYPE_DEFINITIONIMPORT ) 	return ACTTYPE_DEFIMPORT;

		return ACTTYPE_UNKNOWN;
    },
    
    cancelActivity: function(){
		if( this._activity == null ) return false;
		if( this._cancleDeferred != null ) return false;
		
		var actMgr = carter.ui.editors.ActivitiesMgr;
		if( ! actMgr ){
			alert(messages.err_NoActivitiesManger);
			return false;
		}
				
		var ret = false;
		this._cancleDeferred = actMgr.cancelActivity(this._activity.requestId);
			
		if( this._cancleDeferred != null ){			
			this._cancleDeferred.addCallback(this, function(result) {
				ret = result;
				if( this.editor != null )
				{
					if( ret == true )
						this.editor.showOkMsg(this.messages.actCancelOk);
					else
						this.editor.showErrorMsg(this.messages.actCancelFailed);
				}
				if( ret == true){
					this.toolbar.disableButton(0);					
				}
			});
			this._cancleDeferred.addErrback(this, function(error) {
				var errMsg = carter.ui.error.getCarterError(this._cancleDeferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._cancleDeferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._cancleDeferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});	
			this._cancleDeferred.addBoth(this, function() {
				delete this._cancleDeferred;
			});		
		}

		return ret;		
	},
	
	updateState: function(){
		if( this._updateDeferred ) return;
		if( ! this._activity ) return;
		
		// query for data:
		var actMgr = carter.ui.editors.ActivitiesMgr;
		if( ! actMgr ){
			alert(messages.err_NoActivitiesManger);
			return;
		}

		this._updateDeferred = actMgr.getActivity(this._activity.requestId);
		
		if( this._updateDeferred  != null ){
			this._updateDeferred.addCallback(this, function(activity) {
				
				if( this._updateTimer ){				
					clearInterval(this._updateTimer);
					delete this._updateTimer;
				}			
				 				
				if( activity == null ) return;
				this.updateContent(activity, true);
				
			});			
			this._updateDeferred.addErrback(this, function(error) {
				if( this._updateTimer ){
					clearInterval(this._updateTimer);
					delete this._updateTimer;					
				}				
			});
			this._updateDeferred.addBoth(this, function() {
				delete this._updateDeferred;				
			});			
		}
	}    
});
