/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
    dojo.provide("carter.ui.editors.ActivitiesMgr");

    dojo.require("dojo.parser");
    dojo.require("dojox.xml.parser");
    dojo.require("dojo.i18n");

    dojo.require("jazz.app.context");
    dojo.require("jazz.client.xhr");
        
    dojo.require("carter.ui.editors.Definition");
    dojo.require("carter.ui.utils");
    dojo.require("carter.ui.RootServices");
    dojo.requireLocalization("carter.ui", "EditorMessages");
    
    
    var messages = dojo.i18n.getLocalization("carter.ui", "EditorMessages");

    var SERVICE_PATH = carter.ui.RootServices.getUri(carter.ui.Namespaces.CARTER, "activity");
	
	var Utils = carter.ui.utils;
	
	var projectId = jazz.app.context.get().itemId;
	
    carter.ui.editors.ActivitiesMgr = {

    	cancelActivity: function(activityId) {
    		
    		var request = SERVICE_PATH + "?" + Utils.PARAM_REQUESTID + activityId + "&" + Utils.PARAM_PROJID + projectId;

    		var args = {
        			url: request,
                    handleAs: "text",
                	method: "DELETE",
                    load: function(response, ioArgs){
            			return true;
            		}        			
        		};
        		
    		return jazz.client.xhrDelete(args);
        },
 

    	deleteActivity: function(activityId) {
    		
    		var request = SERVICE_PATH + "?" + Utils.PARAM_REQUESTID + activityId + "&" + Utils.PARAM_PROJID + projectId;

    		var args = {
        			url: request,
                    handleAs: "text",
                	method: "DELETE",
                    load: function(response, ioArgs){
            			return true;
            		}        			
        		};
        		
    		return jazz.client.xhrDelete(args);
        },
        
        getActivity: function(activityId) {
        	
        	var getRequest = SERVICE_PATH + "?" + Utils.PARAM_REQUESTID + activityId + "&" + Utils.PARAM_PROJID + projectId;
			var args = {
				url: getRequest,
				headers: {
					"Accept": "application/rdf+json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				load: function(response, ioArgs) {
					var activities = [];
					for (var uri in response) {
						var activity = new carter.ui.editors.Activity();
						activity.parseRdfJson(response[uri]);
						return activity;
					}
				}
			};
			
			return jazz.client.xhrGet(args);
        },

        getActivities: function(itemsLimit) {
        	
			var getRequest = SERVICE_PATH + "?" + Utils.PARAM_PROJID + projectId;
			if( itemsLimit > 0 )
			{
				getRequest += "&" + Utils.PARAM_ITEMSLIMIT + itemsLimit;
			}
			var args = {
				url: getRequest,
				headers: {
					"Accept": "application/rdf+json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				load: function(response, ioArgs) {
					var activities = [];
					for (var uri in response) {
						var activity = new carter.ui.editors.Activity();
						activity.parseRdfJson(response[uri]);
						activities.push(activity);
					}
					
					return activities;
				}
			};
			
			return jazz.client.xhrGet(args);
        }      
    };

})();

