/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
// Client side constants:
var SCHED_DISABLE 			= 0;
var SCHED_ENABLE 			= 1;
var SCHEDTYPE_INTERVAL 		= 1;
var SCHEDTYPE_FIXED			= 2;

(function() {

dojo.provide("carter.ui.editors.Activity");

dojo.require("rmps.core.resource.Resource");
dojo.require("rmps.core.user.UserClient");
dojo.require( "rmps.core.Enum" );

var SCHED_DEFINTERVAL		= 60;

// RDF constants: 
var RDF_ACTTYPE 		= "http://ibm.com/carter/1.0/definitionId";
var RDF_ACTREQUESTER	= "http://ibm.com/carter/1.0/definitionName";
var RDF_ACTREQUESTTIME 	= "http://ibm.com/carter/1.0/modelname";
var RDF_ACTSTARTTIME 	= "http://ibm.com/carter/1.0/schedule";
var RDF_ACTDURATION 	= "http://ibm.com/carter/1.0/scheduleType";
var RDF_STATUS 			= "http://ibm.com/carter/1.0/scheduleInterval";

var RDF_REQUESTER 		= "http://ibm.com/carter/1.0/requester";
var RDF_REQUEST_ID 		= "http://ibm.com/carter/1.0/requestId";
var RDF_ACTIVITY_TYPE 	= "http://ibm.com/carter/1.0/activitytype";
var RDF_START_TIME 		= "http://ibm.com/carter/1.0/startTime";
var RDF_REQUEST_TIME 	= "http://ibm.com/carter/1.0/requestTime";
var RDF_STOP_TIME 		= "http://ibm.com/carter/1.0/stopTime";
var RDF_STATE 			= "http://ibm.com/carter/1.0/state";
var RDF_DETAILS 		= "http://ibm.com/carter/1.0/details";

// Carter reuqirements:
dojo.requireLocalization("carter.ui", "EditorMessages");
dojo.require("carter.ui.utils");

var messages = dojo.i18n.getLocalization("carter.ui", "EditorMessages");

var UserClient = rmps.core.user.UserClient;


carter.ui.editors.Activity.states = new rmps.core.Enum({
	UNKNOWN: new rmps.core.EnumLiteral({name: messages.unknown, value: "unknown", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_unknown.gif")}),
	PENDING: new rmps.core.EnumLiteral({name: messages.pending, value: "pending", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_pending.gif")}),
	IN_PROGRESS: new rmps.core.EnumLiteral({name: messages.inProgress, value: "inProgress", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_inProgress.gif")}),
	ERROR: new rmps.core.EnumLiteral({name: messages.error, value: "error", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_error.gif")}),
	DROPPED: new rmps.core.EnumLiteral({name: messages.dropped, value: "dropped", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_dropped.gif")}),
	COMPLETED: new rmps.core.EnumLiteral({name: messages.completed, value: "completed", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_completed.gif")}),
	WARNING: new rmps.core.EnumLiteral({name: messages.warning, value: "warning", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_warning.gif")}),
	CANCELED: new rmps.core.EnumLiteral({name: messages.canceled, value: "canceled", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_canceled.gif")}),
	CANCELING: new rmps.core.EnumLiteral({name: messages.canceling, value: "canceling", icon: dojo.moduleUrl("carter.ui", "graphics/icons/states/state_canceling.gif")})
});


var States = carter.ui.editors.Activity.states;

dojo.declare("carter.ui.editors.Activity", rmps.core.resource.Resource, {

	stopTime: null, // Date
	startTime: null, // Date
	requestTime: null, // Date
	
	activityType: null,	
	requester: null,
	requestId: null,	
	state: null,
	details: null,
	
	
	contentType: "application/rdf+json",
	
	constructor: function() {	
		this.resetContent();
	},
	
	resetContent: function() {
		this.activityType = null;
		this.requester = null;
		this.requestId = null;
		this.requestTime = null;		
		this.startTime = null;
		this.stopTime = null;
		this.state = null;
		this.details = null;
	},
	
	toRdfJson: function() {
	},

	parseRdfJson: function(rdf){
				
		if( rdf[RDF_STATE] ) {
			this.state = States.get(rdf[RDF_STATE][0].value);
		}
			
		if( rdf[RDF_REQUESTER] ){
			this.requester = rdf[RDF_REQUESTER][0].value;
		}
			
		if( rdf[RDF_REQUEST_ID] ) {
			this.requestId = rdf[RDF_REQUEST_ID][0].value;
		}
		
		if( rdf[RDF_REQUEST_TIME] ) {
			this.requestTime = dojo.date.stamp.fromISOString(rdf[RDF_REQUEST_TIME][0].value);
		}
		
		if (rdf[RDF_START_TIME]) {
			this.startTime = dojo.date.stamp.fromISOString(rdf[RDF_START_TIME][0].value);
		}

		if (rdf[RDF_STOP_TIME]) {
			this.stopTime = dojo.date.stamp.fromISOString(rdf[RDF_STOP_TIME][0].value);
		}

		if (rdf[RDF_ACTIVITY_TYPE]) {
			this.activityType = rdf[RDF_ACTIVITY_TYPE][0].value;
		}	

		if (rdf[RDF_DETAILS]) {
			this.details = rdf[RDF_DETAILS][0].value;
		}	

	},
	
	getRequestTimeStr: function() {
		if( this.requestTime != null ){
			return dojo.date.locale.format(this.requestTime, {formatLength: "short"});
		}		
		return null;
	},
	
	getStartTimeStr: function() {
		if( this.startTime != null ){
			return dojo.date.locale.format(this.startTime, {formatLength: "short"});
		}		
		return null;
	},
	
	getStopTimeStr: function() {
		if( this.stopTime != null ){
			return dojo.date.locale.format(this.stopTime, {formatLength: "short"});
		}		
		return null;
	},
	
	isEnded: function(){
		if(this.state.value == States.ERROR.value || this.state.value == States.DROPPED.value ||
		this.state.value == States.CANCELED.value || this.state.value == States.COMPLETED.value ||
		this.state.value == States.WARNING.value) 
		{
			return true;
		}
		return false;
	},
	
	getRequester: function(){
/*		
		if (this._requester && this._requester.uri == this.requester) {
			var deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._requester);
			}), 0);
			return deferred;
		}
		var deferred = UserClient.getUser(this.requester);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._requester = user;
		}));
		return deferred;
*/
	}	
		
});

})();
