/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
// Client side constants:
var SCHED_DISABLE 			= 0;
var SCHED_ENABLE 			= 1;

var SCHEDTYPE_INTERVAL 		= 1;
var SCHEDTYPE_FIXED			= 2;

var STATCOLOR_INACTIVE		= "#DE0000";
var STATCOLOR_ACTIVE		= "#088639";

(function() {

dojo.provide("carter.ui.editors.Definition");

dojo.require("rmps.core.resource.Resource");
dojo.require("carter.ui.utils");

var Utils = carter.ui.utils;

var SCHED_DEFINTERVAL		= 60;

// RDF constants: 
var RDF_DEFINITION_ID 	= "http://ibm.com/carter/1.0/definitionId";
var RDF_DEFINITION_NAME	= "http://ibm.com/carter/1.0/definitionName";
var RDF_MODELNAME 		= "http://ibm.com/carter/1.0/modelname";
var RDF_SCHEDULE 		= "http://ibm.com/carter/1.0/schedule";
var RDF_SCHED_TYPE 		= "http://ibm.com/carter/1.0/scheduleType";
var RDF_SCHED_INTERVAL 	= "http://ibm.com/carter/1.0/scheduleInterval";
var RDF_SCHED_TIME 		= "http://ibm.com/carter/1.0/scheduleTime";
var RDF_SCHED_DAY 		= "http://ibm.com/carter/1.0/scheduleWeekDay";
var RDF_DEFINITION_STATUS = "http://ibm.com/carter/1.0/status";

dojo.declare("carter.ui.editors.Definition", rmps.core.resource.Resource, {

	name: null,
	defId: null,	
	schedule: null,
	scheduleType: null,
	scheduleInterval: null,
	models: null,
	scheduleTime: null,
	scheduleDays: null,
	statusVal: null,
	statusColor: "#000000", //black
	
	contentType: "application/rdf+json",
	
	constructor: function() {
		this.models = [];
		this.scheduleTime = [];
		this.scheduleDays = [];
		
		this.resetContent();
	},		
	
	setDays: function(days){
		
		while( this.scheduleDays.length > 0 )
			this.scheduleDays.shift();
		
		for(var i = 0 ; i < days.length ; i++){
			this.scheduleDays.push(days[i]);
		}
	},
		
	resetContent: function() {

		this.defId = "new";
		this.name = "";
		this.schedule = SCHED_DISABLE;
		this.scheduleType = SCHEDTYPE_INTERVAL;
		this.scheduleInterval = SCHED_DEFINTERVAL;
		this.setTime(0, 0, false);
		
		while( this.scheduleDays.length > 0 )
			this.scheduleDays.shift();

		while( this.models.length > 0 )
			this.models.shift();

	},
	
	toRdfJson: function() {

		var nodeId = 1;
		
		function createNode() {
			return {
				type: "bnode",
				value: "_:b" + nodeId++
			};
		}
		
		function createInteger(value) {
			if( value == null ) value = 0;
			var result = createLiteral("" + value);
			result.datatype = "http://www.w3.org/2001/XMLSchema#int";
			return result;
		}
		
		function createLiteral(value) {
			return {
				type: "literal",
				value: value
			};
		}
		
		function createUri(value) {
			return {
				type: "uri",
				value: value
			};
		}
		
		var obj = {};
		var models = [];
		var main = {};

		obj[this.uri] = main;
		
		// definition name
		main[RDF_DEFINITION_NAME] = [createLiteral(this.name)];
				
		// definition id
		if( this.defId != "new" ) {
			main[RDF_DEFINITION_ID] = [createLiteral(this.defId)];
		}

		// schedule
		if( this.schedule != null && this.schedule != SCHED_DISABLE){

			if( this.scheduleType == SCHEDTYPE_INTERVAL) {
				main[RDF_SCHEDULE] = [createUri(RDF_SCHED_INTERVAL)];
			}

			else if( this.scheduleType == SCHEDTYPE_FIXED){
				main[RDF_SCHEDULE] = [createUri(RDF_SCHED_TIME)];
			}
			else;
						
			if( this.scheduleInterval != null && this.scheduleInterval.length > 0 ) {
				main[RDF_SCHED_INTERVAL] = [createInteger(this.scheduleInterval)];
			}

			main[RDF_SCHED_TIME] = [createLiteral(this.scheduleTime.join(':'))]; // format as xx:xx 24 hours time representation
				
			if (this.scheduleDays.length > 0) {
				main[RDF_SCHED_DAY] = [];
				for (var i = 0; i < this.scheduleDays.length; ++i) {
					if( this.scheduleDays[i] != null ) {
						main[RDF_SCHED_DAY].push(createInteger(this.scheduleDays[i]));
					}
				}
			}
		}
		
		// models:
		if (this.models.length > 0) {
			main[RDF_MODELNAME] = [];
			for (var i = 0; i < this.models.length; ++i) {
				main[RDF_MODELNAME].push(createLiteral(this.models[i]));
			}
		}
		
		return dojo.toJson(obj);

	},

	parseRdfJson: function(rdf){

		if( rdf == null ){
			return;
		}
		
		// definition name:
		if( rdf[RDF_DEFINITION_NAME] ) {
			this.name = rdf[RDF_DEFINITION_NAME][0].value;
		}
		
		// definition id:
		if(rdf[RDF_DEFINITION_ID]) {
			this.defId = rdf[RDF_DEFINITION_ID][0].value;
		}

		// models
		if ( rdf[RDF_MODELNAME]) {
			var models = rdf[RDF_MODELNAME];
			if (models) {
				for (var i = 0; i < models.length; ++i) {
					var modelName = models[i].value;
					this.models.push(modelName);
				}
			}		
		}
		
		// schedule type:
		if( rdf[RDF_SCHEDULE] ){
			var schedule = rdf[RDF_SCHEDULE][0].value;
			if( schedule && (schedule == RDF_SCHED_INTERVAL || schedule == RDF_SCHED_TIME) ) {
				this.schedule = true;
				if( schedule == RDF_SCHED_INTERVAL )
					this.scheduleType = SCHEDTYPE_INTERVAL;
				else if( schedule == RDF_SCHED_TIME )
					this.scheduleType = SCHEDTYPE_FIXED;
				else;
			}			
		}

		// schedule days
		if( rdf[RDF_SCHED_DAY] ) {
			var days = rdf[RDF_SCHED_DAY];
			if (days) {
				for (var i = 0; i < days.length; ++i) {
					var day = days[i].value;
					this.scheduleDays.push(day);
				}				
			}
		}
		
		// schedule time:
		if (rdf[RDF_SCHED_TIME]) {
			this.scheduleTime = rdf[RDF_SCHED_TIME][0].value.split(':');
		}
		
		// schedule interval:
		if (rdf[RDF_SCHED_INTERVAL]) {
			this.scheduleInterval = rdf[RDF_SCHED_INTERVAL][0].value;
		}
		
		if (rdf[RDF_DEFINITION_STATUS]) {
			var statCode = rdf[RDF_DEFINITION_STATUS][0].value;
			this.setStatus(Utils.getDefStatusDescription(statCode));
			
			if( statCode == Utils.DEFSTAT_ACTIVE)
				this.statusColor = STATCOLOR_ACTIVE;
			else if(statCode == Utils.DEFSTAT_INACTIVE )
				this.statusColor = STATCOLOR_INACTIVE;
		}	
	},
		
	setTime: function(hour, minute, pm){
		if( pm ) {
			hour += 12;
		}
		this.scheduleTime[0] = hour.toString();
		this.scheduleTime[1] = minute.toString();
	},
	
	setStatus: function(statVal){
		this.statusVal = statVal;
	},
	
	getStatus: function() {
		return this.statusVal;
	},
	
	getStatusColor: function() {
		return this.statusColor;
	}
});

})();