/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("carter.ui.editors.DefinitionEditor");

// dojo dependencies:
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.parser");
dojo.require("dojo.io.iframe");
dojo.require("dojo._base.xhr");
dojo.require("dojo.i18n");

// Jazz/Kenton dependencies:
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("rmps.core.RootServices");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.FilterBox");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.MessageBox");
dojo.require("rmps.core.error");

// Carter dependencies:
dojo.require("carter.ui.editors.Definition");
dojo.require("carter.ui.editors.DefinitionsMgr");
dojo.requireLocalization("carter.ui", "EditorMessages");
dojo.require("carter.ui.utils");
dojo.require("carter.ui.error");

// Context help IDs:
var UAKEY_DEFINITION_PAGE = "com.ibm.rational.carter.web.page_DefinitionEditor01";
var UAKEY_MODELS_SEC = "com.ibm.rational.carter.web.section_ImportDefinitionModelsSection01";
var UAKEY_SCHEDULE_SEC = "com.ibm.rational.carter.web.section_ImportDefinitionScheduleSection01";


// globals:
var messages = dojo.i18n.getLocalization("carter.ui", "EditorMessages");

var Utils = carter.ui.utils;

var KEY_RETURN = 13;
var KEY_ENTER = 14;

var connectOnInput = function(obj, widget, method) {
	if (dojo.isMoz || dojo.isOpera) {
		widget.connect(obj, "oninput", method);
	} else {
		widget.connect(obj, "onkeydown", method);
		widget.connect(obj, "onkeyup", method);
		widget.connect(obj, "onpaste", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "oncut", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "onblur", method);
	}
};

// Editor decleration:
dojo.declare("carter.ui.editors.DefinitionEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "carter-ui-editors-DefinitionEditor",
	
	isNew: null,
	itemId: null,
	_definition: null,
	
	_widgetName: null,
	_widgetModels: null,
	_widgetSchedule: null,
	_deferred: null,
	messages: null,

	constructor: function(arguments){
		// set context help id to the editor
		this.uaKey = UAKEY_DEFINITION_PAGE;
		
		this.messages = messages;	
		this.itemId = arguments.itemId;
		if( this.itemId == null ){
			this.itemId = "new";
			this.isNew = true;
		}
		else {
			this.isNew = false;
		}
	},
	
	startup: function() {
		
		delete this._definition;
		this._definition = null;
		
		if( this._widgetModels ){
			this._widgetModels.setPreviewMode();	
		}		
	},
/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2010.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/

    postCreate: function(args) {
        this.inherited(arguments);
        
        if( this._definition == null)
        	this._definition = new carter.ui.editors.Definition();
        else
        	this._definition.resetContent(); 
        
        this.showLoading();
        
        this._init();
       
        this.hideMessage();
    },
    
    _init: function() {
    	// Add hyperlink to definitions page:
    	if( this._headerContent ){
    		var link = dojo.create("a", { href: Utils.getDefinitionsViewUri(), innerHTML: messages.defintions_link});
    		this._headerContent.appendChild(link);	
    	}

    	this._createSections();		
	},
	
	_createSections: function() {
		this._createNameSection();
		this._createModelsListSection();
		this._createScheduleSection();
	},

	_createNameSection: function() {
		this._widgetName = new carter.ui.editors.DefinitionEditor._defName({editor: this});
		if( this._widgetName != null ) {
			this.containerNode.appendChild(this._widgetName.domNode);
			this._widgetName.startup();			
		}
	},
	
	_createModelsListSection: function() {
		this._widgetModels = new carter.ui.editors.DefinitionEditor._ModelsList({editor: this});
		if( this._widgetModels != null ) {
			this.containerNode.appendChild(this._widgetModels.domNode);
			this._widgetModels.startup();
		}	
	},

	_createScheduleSection: function() {
		this._widgetSchedule = new carter.ui.editors.DefinitionEditor._Schedule({editor: this});
		if( this._widgetSchedule != null ) {
			this.containerNode.appendChild(this._widgetSchedule.domNode);
			this._widgetSchedule.startup();
		}		
	},

	addButtons: function(toolbar) {		
		this.inherited(arguments);
		this.refreshButton.domNode.style.display = "none";
	},
	
	setDirty: function() {
		this.saveButton.setEnabled(true);
		this.hideMessage();
	},
	
	save: function() {
		// validate user input:
		if( ! this._validateDataBeforeSave() ) return;
		
		if( this._definition == null )
			this._definition = new carter.ui.editors.Definition(); 
		else
			this._definition.resetContent();
		
		this._collectDefData(this._definition);			
			
		// save definition
		this.saveButton.setEnabled(false);
		
		var defMgr = carter.ui.editors.DefinitionsMgr;
		if( defMgr == null ) return;
		
		var def = dojo.clone(this._definition);
		this._deferred = defMgr.saveDefintion(def);
		
		if( this._deferred != null ){
			
			this._deferred.addCallback(this, function(definition) {
					
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: this.messages.inf_DefinitionSavedOk,
					closable: true
				});
				
				if( this._definition.defId == null || this._definition.defId == "new")
					this._definition.defId = definition.defId;
				if( this._definition.eTag == null )
					this._definition.eTag = definition.eTag;
			
				var uri =  Utils.getDefinitionsViewUri() + "&param1=defSavedOk";
				window.location.href = uri;
			});
			
			this._deferred.addErrback(this, function(error) {
				
				this.saveButton.setEnabled(true);
				
				var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
				if( errMsg != null ){
					this.showErrorMsg(errMsg);
					return;
				}
				
				if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			
			this._deferred.addBoth(this, function() {
				delete this._deferred;
			});		
		}			
	},
	
	showErrorMsg: function(msg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: msg,
			closable: true
		});
	},

	_collectDefData: function(def) {
		
		def.defId = this.itemId;
		
		if( this._widgetName != null ) {
			def.name = this._widgetName.getName();
		}
		if( this._widgetModels != null ) {
			def.models = this._widgetModels.getModels();
		}
		if( this._widgetSchedule != null ) {
			def.schedule 			= this._widgetSchedule.isScheduleEnable();
			def.scheduleType 		= this._widgetSchedule.getScheduleType();
			def.scheduleInterval 	= this._widgetSchedule.getScheduleInterval();
			def.scheduleTime 		= this._widgetSchedule.getScheduleTime();
			def.scheduleDays 		= this._widgetSchedule.getSelectedDays();
		}
	},
	
	getSavedDefinition: function() {
		
		return this._definition;
	},
	
	loadContent: function(itemId) {
		
		this._resetContent();
		
		this.itemId = itemId;
			
		// get definition content from server...
		if( this.itemId != "new" ) {

			var defMgr = carter.ui.editors.DefinitionsMgr;
			if( defMgr != null ) {
				this._deferred = defMgr.getDefinition(this.itemId);
				if( this._deferred != null ){			
					this._deferred.addCallback(this, function(definition) {
						
						if( definition == null ) return;
						delete this._definition;
						this._definition = dojo.clone(definition);

						// populate the page:
						if( this._widgetName != null ) {
							this._widgetName.setName(this._definition.name);
						}
						if( this._widgetModels != null ) {
							this._widgetModels.addModels(this._definition.models);
						}
						if( this._widgetSchedule != null ) {
							this._widgetSchedule.setSchedule(this._definition);
						}
					});
					this._deferred.addErrback(this, function(error) {
						
						var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
						if( errMsg != null ){
							this.showErrorMsg(errMsg);
							return;
						}
						if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
							carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
						} else {
							carter.ui.error.createError(error).dispatch();
						}
					});
					this._deferred.addBoth(this, function() {
						delete this._deferred;
					});
				}
			}
		}		
	},
	
	_resetContent: function() {
		
		this.itemId = "new";
		
		if( this._widgetName != null ) {
			this._widgetName.setName(this.messages.defaultDefinitionName);
		}
		if( this._widgetModels != null ) {
			this._widgetModels.resetContent();
		}
		if( this._widgetSchedule != null ) {
			this._widgetSchedule.resetContent();			
		}
		
		this.hideMessage();
		this.saveButton.setEnabled(false);		 
	},
	
	_validateDataBeforeSave: function() {
	
		// check definition name:
		if( this._widgetName != null ) {
			var defName = this._widgetName.getName();
			if( defName.length == 0 ) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.err_emptyDefinitionName,
					closable: true
				});
				return false;
			}
		}
		
		// check models
		if( this._widgetModels != null ) {
			
			var models = this._widgetModels.getModels();
			
			// check for empty models list:
			if( models == null || models.length == 0 ) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.err_noDefinitionModels,
					closable: true
				});
				return false;
			}
				
			// check for empty model names:
			for(var i=0 ; i < models.length ; i++ ) {
				var name = models[i];
				if( name == null || name.length == 0 ) {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: this.messages.err_noDefinitionModels,
						closable: true
					});
					return false;					
				}
			}
			// check for duplicated names:
			for(var j=0 ; j < (models.length - 1) ; j++) {
				var name1 = models[j];				
				for(var k = (j+1) ; k <= models.length ; k++){
					var name2 = models[k];
					if( name1 == name2){
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: this.messages.err_DuplicatedModelNames,
							closable: true
						});
						return false;
					}
				}
			}
		}
		
		// check schedule
		if( this._widgetSchedule != null ) {
			if( this._widgetSchedule.isScheduleEnable() ){
				var days = this._widgetSchedule.getSelectedDays();
				if( days.length == 0) {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: this.messages.err_noScheduleDay,
						closable: true
					});
					return false;					
				}
				
				if( this._widgetSchedule.getScheduleType() == 0 ) {
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: this.messages.err_noScheduleType,
							closable: true
						});
						return false;								
				}
				
				if( this._widgetSchedule.getScheduleType() == SCHEDTYPE_INTERVAL ) {
					var interval = this._widgetSchedule.getScheduleInterval();
					if( interval == null || interval.length == 0){
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: this.messages.err_noScheduleInterval,
							closable: true
						});
						return false;			
					} 
				} 
			}
		}
				
		return true;
	}

});

// Definition name widget:
dojo.declare("carter.ui.editors.DefinitionEditor._defName", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("carter.ui", "internal/templates/DefinitionNameEditor.html"),
	widgetsInTemplate: true,
	dirty: false,
	editor: null,
	origName: null,
	messages: null,
	
	constructor: function(arguments){
		this.editor = arguments.editor;
		this.messages = messages; 
		this._defIconSrc = dojo.moduleUrl("carter.ui", "graphics/icons/definition.gif");
	},
	
    postCreate: function() {
		this.inherited(arguments);

		this._nameFieldHandlers = [];
		this._nameFieldHandlers.push(this.connect(this.nameEdit, "onkeypress", function (e) {
			if (e.keyCode === KEY_RETURN || e.keyCode === KEY_ENTER) {
				dojo.stopEvent(e);
				if( this.origName != this.nameEdit.value ) {
					this.setDirty(true);
				}
				if( this.nameEdit.value == "") {
					this.setName(this.messages.defaultDefinitionName);
				}
				this.togglePrimaryField("Preview");
				
			}
		}));
		
		this._nameFieldHandlers.push(this.connect(this.previewName,"onmouseover", function (e) {
			e.target.style.backgroundColor = "#FFFFCC";
		}));
		this._nameFieldHandlers.push(this.connect(this.previewName,"onmouseout", function (e) {
			e.target.style.backgroundColor = "";
		}));
		this._nameFieldHandlers.push(this.connect(this.previewName,"onfocus", function (e) {
			this.togglePrimaryField("Edit");
		}));
		this._nameFieldHandlers.push(this.connect(this.nameEdit, "onblur", function (e) {
			this.togglePrimaryField("Preview");
		}));
		
		this.connect(this.nameEdit,"onchange", function (e) {
			this.setDirty(true);
		});
		
		this.setName(this.messages.defaultDefinitionName);
		this.togglePrimaryField("Preview");		
	},

	togglePrimaryField: function (mode) {
		if (mode === "Preview") {
			this.nameEdit.style.display = "none";
			this.previewName.innerHTML = "";
			this.previewName.appendChild(document.createTextNode(this.nameEdit.value || this.messages.defaultDefinitionName));
			this.previewName.style.display = "";
		}
		else if (mode === "Edit") {
			this.previewName.style.display = "none";
			this.nameEdit.style.display = "";
			this.nameEdit.focus();
			this.nameEdit.select();
			this.header.hideMessage();
		}
	},
	
	setName: function(name) {
			this.origName = name;
			this.nameEdit.value = name;
			this.previewName.innerHTML = "";
			this.previewName.appendChild(document.createTextNode(this.nameEdit.value));		
	},
	
	getName: function() {
		return this.nameEdit.value;	
	},
	
	setDirty: function (dirty) {
		
		if (dirty) {
			if( this.editor != null ) {
				this.editor.setDirty();
			}
			//this.modifiedFlag.style.display = "inline";
		} 
		else {
			//this.modifiedFlag.style.display = "none";
		}
		this.dirty = dirty;
	}	
});

// Models list widget:
dojo.declare("carter.ui.editors.DefinitionEditor._ModelsList", [dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	editor: null,
	messages: messages,
	uaKey: UAKEY_MODELS_SEC,
	
	templateString: '<div>' +
		'<p>&nbsp;</p>' + 
		'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" collapsible="false" uaKey="${uaKey}"></div>' +
		'</div>',

	constructor: function(arguments){
		this.editor = arguments.editor;
	},
		
    postCreate: function() {
		this.inherited(arguments); 
		this._loadContent();
	},
	
	_loadContent: function() {
				
		// Pending on previous load:
		if (this._deferred) {
			return;
		}
				
		// Clear previous content...
		
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(this.messages.defModels));				
		this._section.setPrimaryTitle(titleNode);
		
		//create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: this.messages.modelName, width: "100%"},
					{label: this.messages.actions, className: "actions"}
				]
		});
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
				
		// create Add command
		this._section.addAction(this.messages.add, this.messages.add, dojo.hitch(this, function() {
			this._section.edit();
			this._addDefModel();
		}));				
	},
		
	_addDefModel: function(data) {
		var mName = (data != null)? data : this.messages.defaultModelName;
		var newModel = (data != null)? false : true;
		var row = new carter.ui.editors.DefinitionEditor._ModelRow(
			{editor: this.editor, 
			 modelName: mName, 
			 newItem: newModel,
			 list: this});

		if( data == null ) {
			this._table.tbody.insertBefore(row.domNode, this._table.tbody.firstChild);
		}
		else {
			this._table.tbody.appendChild(row.domNode);
		}

		if(! data ) {
			row.focus();
			this.editor.setDirty();
		}
	},
	
	addModels: function(models) {
		if( models != null ) {
			for(var idx = 0 ; idx < models.length ; idx ++ ) {
				this._addDefModel(models[idx]);
			}
		}
	},
	
	resetContent: function() {
	
		var count = 0;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			row.destroyRecursive();
			delete row;
			count += 1;
		});
		
		dojox.xml.parser.removeChildren(this._table.tbody);
	},
	
	getModels: function() {
		var models = [];	
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			models.push(row.getName());
		});
		
		return models;
	},
	
	setPreviewMode: function(){
		this._section.preview();
	}
});

dojo.declare("carter.ui.editors.DefinitionEditor._ModelRow", [dijit._Widget, dijit._Templated], {

    widgetsInTemplate: true,
    templateString: '<tr>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_modelNamePreview"></span>' +
								'<input class="editable" style="width: 50%;" dojoAttachPoint="_modelNameEdit" value="" type="text"></input>' +
							'</div>' +
						'</td>' +
						'<td class="actions" style="width: 50px;">' +
							'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
						'</td>' +
					'</tr>',

    messages: messages,
    editor: null,

    _modelName: null,
    _deleteModelDeferred: null,

    _popupContent: null,
    _popup: null,
    _actionBar: null,
    _newItem: null,
    _list: null,

    constructor: function(arguments) {
        this.editor = arguments.editor;
        this._modelName = arguments.modelName;
        this._newItem = arguments.newItem;
        this._list = arguments.list;
    },

    postCreate: function() {
    	var isNewRow = true;
        this._modelNameEdit.value = this._modelName;
        this._modelNamePreview.innerHTML = "";
		this._modelNamePreview.appendChild(dojo.doc.createTextNode(this._modelName));
		
        this.toolbar.addButton(this, function(e) {
            dojo.stopEvent(e);
            this._deleteModel();
            this.destroyRecursive();
        }, {
            label: this.messages.msgDelete,
            imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/remove.gif")
        });

        connectOnInput(this._modelNameEdit, this, function() {
            this._modelName = dojo.trim(this._modelNameEdit.value);
            this.editor.setDirty();
            this._modelNamePreview.innerHTML = "";
			this._modelNamePreview.appendChild(dojo.doc.createTextNode(this._modelName));        
        });
		
		this.connect(this.name, "onkeypress", function (e) {
			if (e.keyCode === KEY_RETURN || e.keyCode === KEY_ENTER) {
				dojo.stopEvent(e);
				if( this._list ) this._list.setPreviewMode();
				if( this.editor) this.editor.focus();
			}
		});
		
		if( this._newItem && this._newItem == true ) {
        	this._modelNameEdit.focus();
        }
    },

    focus: function() {
        this._modelNameEdit.focus();
    },

    setName: function(name) {
        this._modelName = name;
        this._modelNameEdit.value = this._modelName; 
    },
	
	getName: function() {
		return this._modelName; 
	},
	
    destroy: function() {
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
            delete this._popupContent;
        }
        if (this._popup) {
            this._popup.destroyRecursive();
            delete this._popup;
        }
        this.inherited(arguments);
    },

    _deleteModel: function() {
    },

    _updateActions: function() {

        this._actionBar.addButton(this, function(e) {
            dojo.stopEvent(e);
            this._deleteModel();
            this.destroyRecursive();
            if (this.editor != null)
                this.editor.setDirty();
        }, {
            label: messages.msgDelete,
            imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/remove.gif")
        });
    },

    _createPopupContents: function() {
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
        }
    },    
});

// Schedule widget:
dojo.declare("carter.ui.editors.DefinitionEditor._Schedule", [dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	editor: null,
	_section: null,
	_definition: null,
	_modifiedDefinition: null,
	messages: messages,
	uaKey: UAKEY_SCHEDULE_SEC,

	templateString: '<div><p>&nbsp;</p>' +
						'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="false" uaKey="${uaKey} primaryTitle="${messages.schedule}"">' +
							'<div class="schedule">' +
								'<div class="item">' +
									'<br>' +
									'<input id="${id}_eCb" type="checkbox" dojoAttachPoint="_enabledCb" dojoAttachEvent="onclick: _onEnabled">' +
									'<label for="${id}_eCb">${messages.enableSchedule}</label>' +
									'<br><br>' +
								'</div>' +
								'<fieldset style="float: left;">' +
									'<legend>${messages.importTime}</legend>' +
									'<br>' +
										'<div class="item">' +
											'<input dojoAttachPoint="_intervalRadio" type="radio" name="scheduleType" value="interval" dojoAttachEvent="onclick: _onInterval"> ${messages.interval} ' +
											'<input dojoAttachPoint="_intervalInput" type="text" maxlength="4" style="width: 50px;">' +
											'<br><br>' +
											'<input dojoAttachPoint="_specificRadio" type="radio" name="scheduleType" value="specific" dojoAttachEvent="onclick: _onSpecific"> ${messages.at} ' +
											'<select dojoAttachPoint="_hoursSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> : ' +
											'<select dojoAttachPoint="_minutesSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> ' +
											'<select dojoAttachPoint="_ampmSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> ${messages.gmt}' + 
										'</div>' +
								'<br><br>' +
								'</fieldset>' +
								'<fieldset style="float: left; margin-left: 20px;">' +
									'<legend>${messages.importDays}</legend>' +
									'<div class="item">' +
										'<select dojoAttachPoint="_daysSelect" dojoAttachEvent="onchange: _onDaysChanged" multiple="true" size="7" style="margin-right: 10px; overflow: hidden; float: left;"></select>' +
										'<div style="float: left;">' +
											'<button dojoAttachPoint="_selectAllBtn" style="width: 100%; margin-bottom: 4px;" dojoAttachEvent="onclick: _onSelectAllDays">${messages.selectAll}</button><br>' +
											'<button dojoAttachPoint="_deselectAllBtn" style="width: 100%;" dojoAttachEvent="onclick: _onDeselectAllDays">${messages.deselectAll}</button>' +
										'</div>' +
										'<div style="clear: both;"></div>' + // clearing div
									'</div>' +
								'</fieldset>' +
								'<div style="clear: both;"></div>' + // clearing div
							'</div>' +
						'</div>' +
					'</div>',

	
	postCreate: function() {
		if (dojo.isIE < 8) {
			this._selectAllBtn.style.width = "";
			this._deselectAllBtn.style.width = "";
		}
		
		if( this.editor == null ) return;
		
		this._definition = this.editor._definition;
		this._ModifiedDefinition = dojo.clone(this._definition);
		

		// populate hours
		for (var i = 1; i <= 12; ++i) {
			var label = i + "";
			if (i < 10) {
			 	label = "0" + i;
			}
			var value = label;
			if (i === 12) {
				value = "00";
			}
			this._createOption(this._hoursSelect, label, value, false);
		}
		
		// populate minutes
		for (var i = 0; i < 60; ++i) {
			var value = i;
			if (i < 10) {
			 	value = "0" + i;
			}
			this._createOption(this._minutesSelect, value, value, false);
		}
		
		// populate AM / PM
		var isPM = false;
		this._createOption(this._ampmSelect, dojo.date.locale._getGregorianBundle()["am"], "am", !isPM);
		this._createOption(this._ampmSelect, dojo.date.locale._getGregorianBundle()["pm"], "pm", isPM);
		
		// populate week days
		var weekDays = dojo.date.locale.getNames("days", "wide");
		
		// move sunday to the end
		weekDays.push(weekDays[0]);
		weekDays.shift();
		
		for (var i = 0; i < weekDays.length; ++i) {
			this._createOption(this._daysSelect, weekDays[i], i + 1, false);
		}
		
		// disable schedule:
		this._updateEnablement( false );
		
		// Add event handlers:
		connectOnInput(this._intervalInput, this, function() {
			this._onIntervalChanged();
        });
		
	},
	
	startup: function() {

		this.inherited(arguments);
		if (dojo.isIE < 8) {
			var sow = this._selectAllBtn.offsetWidth;
			var dow = this._deselectAllBtn.offsetWidth;
			if (sow < dow) {
				this._selectAllBtn.style.width = dow + "px";
			} 
			else {
				this._deselectAllBtn.style.width = sow + "px";
			}
		}

	},
	
	_createOption: function(selectNode, label, value, selected) {

		var option = dojo.create("option");
		option.value = value;
		option.appendChild(dojo.doc.createTextNode(label));
		if (selected) {
			option.selected = true;
		}
		selectNode.appendChild(option);

	},
	
	_updateEnablement: function(enabled) {

		if (enabled) {
			this._section.setSecondaryTitle(messages.scheduleEnabled);
			
			if (dojo.isIE < 8 || dojo.isChrome) {
				var options = this._daysSelect.options;
				for (var i = 0, option; option = options[i]; ++i) {
					if (option.selected) {
						dojo.removeClass(option, "disabledSelected");
					}
				}
			}
			
			this._intervalRadio.disabled = false;
			this._specificRadio.disabled = false;
			this._daysSelect.disabled = false;
			this._selectAllBtn.disabled = false;
			this._deselectAllBtn.disabled = false;
			
			if (this._intervalRadio.checked) {
				this._intervalInput.disabled = false;
				this._hoursSelect.disabled = true;
				this._minutesSelect.disabled = true;
				this._ampmSelect.disabled = true;
			} else {
				this._intervalInput.disabled = true;
				this._hoursSelect.disabled = false;
				this._minutesSelect.disabled = false;
				this._ampmSelect.disabled = false;
			}
			var TimeSelected = (this._intervalRadio.checked || this._specificRadio.checked)? true : false;
			if( ! TimeSelected ){
				this._specificRadio.checked = true;
				this._hoursSelect.disabled = false;
				this._minutesSelect.disabled = false;
				this._ampmSelect.disabled = false;			
			}
		} else {
			this._section.setSecondaryTitle(messages.scheduleDisabled);
			this._intervalRadio.disabled = true;
			this._specificRadio.disabled = true;
			this._intervalInput.disabled = true;
			this._hoursSelect.disabled = true;
			this._minutesSelect.disabled = true;
			this._ampmSelect.disabled = true;
			this._daysSelect.disabled = true;
			this._selectAllBtn.disabled = true;
			this._deselectAllBtn.disabled = true;
			
			if (dojo.isIE < 8 || dojo.isChrome) {
				var options = this._daysSelect.options;
				for (var i = 0, option; option = options[i]; ++i) {
					if (option.selected) {
						dojo.addClass(option, "disabledSelected");
					}
				}
			}
		}
	},
	
	_onEnabled: function() {		

		this._updateEnablement(this._enabledCb.checked);
		this.editor.setDirty();
	},
	
	_onInterval: function() {
		this._updateEnablement(true);
		this.editor.setDirty();
	},
	
	_onSpecific: function() {
		this._updateEnablement(true);
		this.editor.setDirty();
	},
	
	_onSelectAllDays: function() {

		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = true;
		}
		this._onDaysChanged();

	},
	
	_onDeselectAllDays: function() {

		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = "";
		}
		this._onDaysChanged();

	},
	
	_onIntervalChanged: function() {
		this.editor.setDirty();
	},
	
	_onScheduleTimeChanged: function() {
		this.editor.setDirty();
	},
		
	_onDaysChanged: function() {				
		this.editor.setDirty();
	},
	
	_parseHour: function(hour) {
		return parseInt(hour.replace(/^0/, ''));
	},
	
	resetContent: function() {
		
		// deselect days:
		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = "";
		}
		
		this._enabledCb.checked = false;
		this._updateEnablement(false);
		
	},
	
	setSchedule: function(definition) {
		
		this.resetContent();
		
		if( definition == null ) return;
		if( definition.schedule == SCHED_DISABLE ) return;
		
		
		if (definition.scheduleType == SCHEDTYPE_INTERVAL ) {
			this._intervalRadio.checked = true;
		}
		else if( definition.scheduleType == SCHEDTYPE_FIXED ) {
			this._specificRadio.checked = true;
		}
		
		this._intervalInput.value = definition.scheduleInterval;
		

		//	********* Select time ************* 
		var nHour = parseInt(definition.scheduleTime[0]);
		var isPm = false;

		if( nHour > 12 ){
			isPm = true;
			nHour -= 12;
		}
		
		var amPmItemId = (isPm)? 1 : 0;
		var amPmItem = this._ampmSelect.options[amPmItemId];
		amPmItem.selected = true;
		
		var option = this._hoursSelect.options[nHour - 1];
		if( option != null ){
			option.selected = true;
		}

		var nMin = parseInt(definition.scheduleTime[1]);		
		var option = this._minutesSelect.options[nMin];
		if( option != null ){
			option.selected = true;
		}

		//	********* Select days *************
		// deselect all days first 
		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = "";
		}
		
		for( var i = 0 ; i < definition.scheduleDays.length ; i++ ) {
			var day = definition.scheduleDays[i];
			if( day > 0 && day <= this._daysSelect.options.length ) {
				var option = this._daysSelect.options[day-1];
				if ( option != null )
					option.selected = true;
			} 
		}
		
		this._enabledCb.checked = definition.schedule;
		
		this._updateEnablement(this._enabledCb.checked);		
	},
	
	getSelectedDays: function() {
		var days = [];
		var options = this._daysSelect.options;
		for (var i = 0, option ; option = options[i] ; ++i) {
			if (option != null && option.selected) {
				days.push(i + 1);
			}
		}
		return days;
	},
	
	isScheduleEnable: function() {
		if( this._enabledCb.checked )
			return SCHED_ENABLE;
		else 
			return SCHED_DISABLE; 
	},
	
	getScheduleType: function() {
		if( this._intervalRadio.checked ) 		return SCHEDTYPE_INTERVAL;
		else if( this._specificRadio.checked ) 	return SCHEDTYPE_FIXED;
		else 									return 0;
	},
	
	getScheduleInterval: function() {
		return this._intervalInput.value;
	},
	
	getScheduleTime: function() {
		var hour = this._hoursSelect.options[this._hoursSelect.selectedIndex].value;
		var minutes = this._minutesSelect.options[this._minutesSelect.selectedIndex].value;
		if (this._ampmSelect.options[this._ampmSelect.selectedIndex].value === "pm") {
			hour = this._parseHour(hour) + 12;
		}

		var time = [hour, minutes];
		return time;
	}
});

