/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2010.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/
// declare the Dojo module
dojo.provide("carter.ui.editors.DefinitionsEditor");

// Dojo requirements
dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.io.iframe");

//jazz/kenton requirements
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.context");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.StyledTable");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.FilterBox");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.ui.EditableCollapsibleSection");
// Carter reuqirements:
dojo.requireLocalization("carter.ui", "EditorMessages");
dojo.require("carter.ui.editors.DefinitionsMgr");
dojo.require("carter.ui.utils");
dojo.require("carter.ui.error");

// globals:
var messages = dojo.i18n.getLocalization("carter.ui", "EditorMessages");

var Utils = carter.ui.utils;

// Context help IDs:
var UAKEY_DEFINITIONS_PAGE = "com.ibm.rational.carter.web.page_DefinitionsEditor01";

// Editor implementation:
dojo.declare("carter.ui.editors.DefinitionsEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "carter-ui-editors-definitionsEditor",

	defList: null,
	
	constructor: function() {
		// set context help id to the editor
		this.uaKey = UAKEY_DEFINITIONS_PAGE;
	},
	
    postCreate: function() {
        this.inherited(arguments);
        this._init();
    },
    
    startup: function(){
		this.hideMsg();
	},
    
    _init: function() {		
		this._createSections();
	},
	
	_createSections: function() {
		this._createMainSection();
	},

	_createMainSection: function() {
		this.defList = new carter.ui.editors.DefinitionsEditor._defList({editor: this});
		this.containerNode.appendChild(this.defList.domNode);
		this.defList.startup();		
	},
	
	addButtons: function(toolbar) {		
		this.inherited(arguments);

		// hide page buttons (top-right corner) 
		this.saveButton.domNode.style.display = "none";
		
		toolbar.addWidget(new rmps.ui.SimpleButton({
				label: messages.addDefinition,
				onClick: dojo.hitch(this, this.createNewDef)
			}));		
	},
	
	addDefinition: function(definition) {
		this.hideMsg();
		if( definition != null && this.defList != null ) { 
			this.defList.addOrUpdateDifinition(definition.name, definition.defId);
		}
	},
	
	createNewDef: function() {
		if( this.defList != null ){
			this.defList._OnAddDefinition(null);
		}
	},
	
	findDefName: function(defName){
		if( this.defList ) {
			return this.defList.findDefinition(defName);
		}
		else
			return false;
	},
	
	refresh: function() {
		if( this.defList ) {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.loading
			});

			this.defList.loadContent();
			
			this.hideMessage();
		}
	},
	
	showInfoMsg: function(msg){
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: msg,
			closable: true
		});
	},
	
	showOkMsg: function(msg){
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: msg,
			closable: true
		});
	},

	showErrorMsg: function(msg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: msg,
			closable: true
		});
	},

	hideMsg: function(){
		this.hideMessage();
	}
	
});


// Widget implementation:
dojo.declare("carter.ui.editors.DefinitionsEditor._defList", [dijit._Widget, dijit._Templated], {
	
	messages: null,
	widgetsInTemplate: true,
	editor: null,
	_updateTimer: null,
	_deferred: null,
	_defStatusTimer: null,
	_defStatusUpdateDeferred: null,
  	
	templateString: '<div>' +
		'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="false""></div>' +
		'</div>',

	constructor: function(arguments){
		this.editor =  arguments.editor;
		this.messages = messages;
	},
	
    postCreate: function() {
		this.inherited(arguments);
				 
		 //create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: this.messages.defName, width: "45%"},
					{label: this.messages.defStatus, width: "40%"},
					{label: this.messages.actions, width: "15%"}
				]
		});
		
		this._section.headerNode.style.display = "none";
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
								
		this.loadContent();
		
		// start update timers: 
		if( ! this._updateTimer ){
			this._updateTimer = setInterval(dojo.hitch(this, this.loadContent), Utils.TIMEINTERVAL_DEFINITIONS_REFRESH);
		}
		
		if( ! this._defStatusTimer ){
			this._defStatusTimer = setInterval(dojo.hitch(this, this.refreshStatus), Utils.TIMEINTERVAL_DEF_STATUS);
		}
		
	},
	
	destroy: function() {
		this.cancelStatusRefresh();
		
   		if (this._updateTimer) {
			clearInterval(this._updateTimer);
			delete this._updateTimer;
		} 	
		if (this._deferred) {
			this._deferred.cancel();
			delete this._deferred;
		}
        this.inherited(arguments);
    },
	
	loadContent: function() {
			
		// pervent concurrent requests:
		if (this._deferred) {
			return;
		}
		
		if( this.editor != null )
			this.editor.showInfoMsg(this.messages.loading);
			
		// Clear previous content...
		this.resetContent();
		
		// query for data:
		var DefMgr = carter.ui.editors.DefinitionsMgr;
		if( ! DefMgr ){
			alert(messages.err_NoDefinitionManger);
			return;
		}
				
		var projectId = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId;
		this._deferred = DefMgr.getDefintions(projectId);
		
		// Populate the table:
		if( this._deferred != null ){			
			this._deferred.addCallback(this, function(definitions) {
				
				if( definitions == null ) return;
				
				function defCompare(a, b) {
					var aLower = a.name.toLowerCase();
					var bLower = b.name.toLowerCase();
					return aLower < bLower ? -1 : 1;
				}
				
				definitions.sort(defCompare);
				
				for( var i = 0 ; i < definitions.length; ++i) {
					this.addDefToList(definitions[i].name, definitions[i].defId);
				}
			});
			this._deferred.addErrback(this, function(error) {				
				var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			this._deferred.addBoth(this, function() {
				delete this._deferred;
			});
		}
		
		if( this.editor != null )
			this.editor.hideMsg();
		
		this.refreshStatus();		
	},
	
	refreshStatus: function(){
		
		// if in the middle of other task:
		if ( this._defStatusUpdateDeferred ) {
			return;
		}
				
		// query for data:
		var DefMgr = carter.ui.editors.DefinitionsMgr;
		if( ! DefMgr ){
			alert(messages.err_NoDefinitionManger);
			return;
		}
		
		this._defStatusUpdateDeferred = DefMgr.getDefintionsStatus();
		
		// Populate the table:
		if( this._defStatusUpdateDeferred != null ){			
			this._defStatusUpdateDeferred.addCallback(this, function(definitions) {

				if( definitions == null ) return;
				
				for( var i = 0 ; i < definitions.length; ++i) {
					var defId = definitions[i].defId;
					var defRow = this._findRowByDefId(definitions[i].defId);
					if( defRow != null){ 
						defRow.setStatus(definitions[i].getStatus(), definitions[i].getStatusColor());
					}
				}
			});
			this._defStatusUpdateDeferred.addErrback(this, function(error) {
				// do nothing on error.				
			});
			this._defStatusUpdateDeferred.addBoth(this, function() {
				delete this._defStatusUpdateDeferred;
			});
		}	
	},
	
	cancelStatusRefresh: function(){
		
		if (this._defStatusTimer) {
			clearInterval(this._defStatusTimer);
			delete this._defStatusTimer;
		} 	
		if (this._defStatusUpdateDeferred) {
			this._defStatusUpdateDeferred.cancel();
			delete this._defStatusUpdateDeferred;
		}
	},
	
	_OnAddDefinition: function(data) {
		var uri =  Utils.getDefinitionEditorUri() + "&itemId=new";
		window.location.href = uri;	
	}, 
	
	addDefToList: function(defName, defId) {
		var row = new carter.ui.editors.Definitions._DefinitionRow({list: this});
		this._table.tbody.appendChild(row.domNode);
		row.setName(defName);
		row.setId(defId);
		
		this.refreshStatus();
	},
	
	addOrUpdateDifinition: function(defName, defId){
		var row = this._findRowByDefId(defId)
		if( row != null && row.getId() == defId ) {
			row.setName(defName);
		}
		else {
			this.addDefToList(defName, defId);
		}
	},
	
	_findRowByDefId: function(defId){
		var retRow = null;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			if( row.getId() == defId ) {
				retRow = row;
			}
		});
		return retRow;		
	},
	
	EditDefinition: function(defId) {
		var uri =  Utils.getDefinitionEditorUri() + "&itemId=" + defId;
		window.location.href = uri;	
	},

	RunDefinition: function(defId) {
			
		if( defId == null ) return false;
		var DefMgr = carter.ui.editors.DefinitionsMgr;
		if( ! DefMgr ) return false;

		this._runDefinitionRet = false;
		
		this._deferred = DefMgr.runDefinition(defId);
		
		if( this._deferred != null ){			
			this._deferred.addCallback(this, function(result) {
				var retCode = result; 
				if( this.editor != null )
				{
					if( retCode == Utils.RETVAL_COMPLETEDOK ){
						var message = dojo.string.substitute(this.messages.inf_ScheduleDefOk, [Utils.getActivitiesEditorUri() + "&refresh=true"]);
						this.editor.showOkMsg(message);
						this._runDefinitionRet = true;
					}
					else {
						this.editor.showErrorMsg(this.messages.err_ScheduleDefFailed);
						this._runDefinitionRet = false;
					}
				}
			});
			this._deferred.addErrback(this, function(error) {
				var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			this._deferred.addBoth(this, function() {
				delete this._deferred;
			});			
		}

		return this._runDefinitionRet;

	},
	
	DeleteDefinition: function(defId) {
		
		var DefMgr = carter.ui.editors.DefinitionsMgr;
		if( ! DefMgr ){
			alert(messages.err_NoDefinitionManger);
			return false;
		}
		this._deletedDefId = defId;
		
		var ret = false;
		this._deferred = DefMgr.deleteDefinition(defId);
		
		if( this._deferred != null ){			
			this._deferred.addCallback(this, function(result) {
				ret = result;
				if( this.editor != null )
				{
					if( ret == true )
						this.editor.showOkMsg(this.messages.inf_DefDeletedOk);
					else
						this.editor.showErrorMsg(this.messages.err_DefDeletionFailed);
				}
				if( ret == true){
					this.removeDefFromList(this._deletedDefId);
				}
			});
			this._deferred.addErrback(this, function(error) {
				var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});	
			this._deferred.addBoth(this, function() {
				delete this._deferred;
			});		
		}

		return ret;
	},

	findDefinition: function(defName){
		
		var ret = false;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			if( row.getName() == defName ) {
				ret = true;
			}
		});
		return ret;
	},
	
	removeDefFromList: function(defId){
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			if( row.getId() == defId ) {
				row.destroyRecursive();
				delete row;
			}
		});		
	},

	resetContent: function() {
		
		var count = 0;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			row.destroyRecursive();
			delete row;
			count += 1;
		});
		
		dojox.xml.parser.removeChildren(this._table.tbody);
	}
});

dojo.declare("carter.ui.editors.Definitions._DefinitionRow", [dijit._Widget, dijit._Templated], {
    widgetsInTemplate: true,
    templateString: '<tr>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_defNamePreview"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span dojoAttachPoint="_defStatus"></span>' +
							'</div>' +
						'</td>' +	
						'<td class="actions" style="width: 200px;">' +
							'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
						'</td>' +
					'</tr>',

    messages: messages,
    list: null,

    _definitionName: null,
    _definitionId: null,
    _deleteModelDeferred: null,

    _popupContent: null,
    _popup: null,
    _actionBar: null,
    _defStatusVal: null,

    constructor: function(arguments) {
        this.list = arguments.list;
    },

    postCreate: function() {
    	this.setName("");

        this.toolbar.addButton(this, function(e) {
            dojo.stopEvent(e);
            this._editDefinition();
        }, {
            label: this.messages.msgEdit,
            imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/edit.gif")
        });

        this.toolbar.addButton(this, function(e) {
            dojo.stopEvent(e);
            this._runDefinition();
        }, {
            label: this.messages.scheduleImport,
            imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/request.gif")
        });

        this.toolbar.addButton(this, function(e) {
            dojo.stopEvent(e);
            if( this._deleteDefinition() )
            	this.destroyRecursive();
            else {
            	// show error message to the user
            }
        }, {
            label: this.messages.msgDelete,
            imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/remove.gif")
        });        
    },

    setName: function(name) {
        this._definitionName = name;
        this._defNamePreview.innerHTML = "";
        this._defNamePreview.appendChild(dojo.doc.createTextNode(this._definitionName)); 
    },
	
	getName: function() {
		return this._definitionName; 
	},
	
	setId: function(itemId){
		this._definitionId = itemId;	
	},
	
	getId: function() {
		return this._definitionId;
	},
	
    setStatus: function(defStatus, defStatusColor) {
        this._defStatusVal = defStatus;
        this._defStatus.innerHTML = "";
        this._defStatus.appendChild(dojo.doc.createTextNode(this._defStatusVal));
        this._defStatus.style.color = defStatusColor;
    },
	
	getStatus: function() {
		return this._defStatusVal; 
	},

    destroy: function() {
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
            delete this._popupContent;
        }
        if (this._popup) {
            this._popup.destroyRecursive();
            delete this._popup;
        }
        this.inherited(arguments);
    },

    _deleteDefinition: function() {
    	var ret = false;
		if( this.list ){
			ret = this.list.DeleteDefinition(this.getId());
		}
		return ret;
    },

    _runDefinition: function() {
    	var ret = false;
		if( this.list ){
			ret = this.list.RunDefinition(this.getId());
		}
		return ret;
    },

	_editDefinition: function(){
		if( this.list ){
			this.list.EditDefinition(this.getId());
		}
	},

    _createPopupContents: function() {
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
        }
    }

});
