/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
    dojo.provide("carter.ui.editors.DefinitionsMgr");

    dojo.require("dojo.parser");
    dojo.require("dojox.xml.parser");
    dojo.require("dojo.i18n");

    dojo.require("jazz.app.context");
    dojo.require("jazz.client.xhr");
        
    dojo.require("carter.ui.editors.Definition");
    dojo.require("carter.ui.utils");
    dojo.require("carter.ui.RootServices");
    dojo.requireLocalization("carter.ui", "EditorMessages");
    
    
    var messages = dojo.i18n.getLocalization("carter.ui", "EditorMessages");

    var SERVICE_PATH = carter.ui.RootServices.getUri(carter.ui.Namespaces.CARTER, "definitions");
	
	var Utils = carter.ui.utils;
	
	var projectId = jazz.app.context.get().itemId;
	
    carter.ui.editors.DefinitionsMgr = {

    	deleteDefinition: function(defintionId) {
    		    		
    		var delRequest = SERVICE_PATH + "?" + Utils.PARAM_PROJID + projectId + "&" + Utils.PARAM_DEFID + defintionId;
    		
    		var args = {
        			url: delRequest,
                    handleAs: "text",
                	method: "DELETE",
                    error: function(response, ioArgs){
            			return false;
            		},
                    load: function(response, ioArgs){
            			return true;
            		}        			
        		};
        		
    		return jazz.client.xhrDelete(args);
        },
        
        getDefintions: function(projectId) {
		
			var getRequest = SERVICE_PATH + "?" + Utils.PARAM_PROJID + projectId;
		        	
			var args = {
				url: getRequest,
				headers: {
					"Accept": "application/rdf+json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				load: function(response, ioArgs) {
					var definitions = [];
					var ProjArea = response[projectId];
					if( ProjArea ) {
						var defs = ProjArea["Definition"];
						for (var i = 0; i < defs.length; ++i) {
							var val = defs[i].value;
							var definition = new carter.ui.editors.Definition();
							definition.parseRdfJson(response[val]);
							definitions.push(definition);
						}						
					}
					
					return definitions;
				}
			};
			
			return jazz.client.xhrGet(args);
        },
        
        getDefintionsStatus: function() {
		
			var getRequest = SERVICE_PATH + "?" + Utils.PARAM_PROJID + projectId + "&" + Utils.PARAM_REQUESTID + Utils.REQTYPE_DEFSTATUS;
		        	
			var args = {
				url: getRequest,
				headers: {
					"Accept": "application/rdf+json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				load: function(response, ioArgs) {

					var definitions = [];
					for (var uri in response) {
						var definition = new carter.ui.editors.Definition();
						definition.parseRdfJson(response[uri]);
						definitions.push(definition);
					}
					return definitions;
				}
			};
			
			return jazz.client.xhrGet(args);
        },

        getDefinition: function(definitionId) {
		
			var getRequest = SERVICE_PATH + "?" + Utils.PARAM_DEFID + definitionId;
		    
			var args = {
				url: getRequest,
				headers: {
					"Accept": "application/rdf+json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				load: function(response, ioArgs) {
					var def = null;
					for(var uri in response){
						if( uri){
							def = new carter.ui.editors.Definition();		
							def.parseRdfJson(response[uri]);
							def.eTag = ioArgs.xhr.getResponseHeader(Utils.PARAM_ETAG);
							return def;
						}
					}
				}
			};
			
			return jazz.client.xhrGet(args);
        },
        
        runDefinition: function(definitionId) {
			
			var getRequest = SERVICE_PATH + "?" + Utils.PARAM_DEFID + definitionId + "&" + Utils.PARAM_REQUESTID + Utils.REQTYPE_EXECUTEDEF;
			var args = {
				url: getRequest,
				headers: {
					"Accept": "application/rdf+json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				load: function(response, ioArgs) {
					return ioArgs.xhr.getResponseHeader(Utils.PARAM_RETCODE);
				}
			};
			
			return jazz.client.xhrGet(args);
        },

        saveDefintion: function(definition) {
	
			if (definition == null) return;
			
			
			var saveRequest = SERVICE_PATH + "?" + Utils.PARAM_PROJID + projectId + "&" + Utils.PARAM_DEFID + definition.defId;					
			var args = {
				headers: {
					"Content-Type": definition.contentType,
					"If-Match": definition.eTag,
					"X-ibm-rmps-internal": "true"
				},
				url: saveRequest,
				hasBody: true,
				postData: definition.toRdfJson(),
				
				load: dojo.hitch(this, function(response, ioArgs) {
					definition.eTag = ioArgs.xhr.getResponseHeader(Utils.PARAM_ETAG);
					definition.defId = ioArgs.xhr.getResponseHeader(Utils.PARAM_DEFID_SHORT);						 
					return definition;  
				})
			};
			return jazz.client.xhrPost(args);
        }
    };

})();
