/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2009.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/

(function() {
    dojo.provide("carter.ui.error");

    dojo.require("jazz.util.html");

    dojo.require("jazz.ui.StyledTable");
    dojo.require("dojox.xml.DomParser");

    dojo.require("dojo.i18n");
    dojo.require("dojo.string");
    dojo.require("jazz.util.date");

    dojo.requireLocalization("carter.ui", "ErrorMessages");
	dojo.require("carter.ui.utils");
	
    var errorMessages = dojo.i18n.getLocalization("carter.ui", "ErrorMessages");

    carter.ui.error = {

        Severity: {
            INFO: "INFO",
            WARNING: "WARNING",
            ERROR: "ERROR",
            CRITICAL: "CRITICAL"
        },

        create: function(message, details, action) {
            return new carter.ui.Error({
                "message": message,
                "detailedMessage": details,
                action: action,
                _isGeneric: false
            });
        },

        createError: function(error, action) {
            return new carter.ui._Error(error, action);
        },
		
		getRdfTagValue: function(ioArgs, tagName) {
			var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
            if (ioArgs.xhr.responseText && contentType && contentType.toLowerCase() === "application/rdf+xml") {
                var rdf = dojox.xml.DomParser.parse(ioArgs.xhr.responseText);
                var rdfParser = new carter.ui._RdfError(ioArgs, rdf.getElementsByTagName("rdf:Description")[0], null);
                if( rdfParser != null ) {
                	return rdfParser.getTagValue(tagName);
                }
            }
            return null;
		},
		
		getCarterError: function(ioArgs) {
			var errCode = carter.ui.error.getRdfTagValue(ioArgs, "errorMessage");
			if( errCode != null ){ 
				return carter.ui.utils.getErrorDescription(errCode);
			}
			return null;
		},
		
        createXhrError: function(ioArgs, action) {
            var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
            if (ioArgs.xhr.responseText && contentType && contentType.toLowerCase() === "application/rdf+xml") {
                var rdf = dojox.xml.DomParser.parse(ioArgs.xhr.responseText);
                return new carter.ui._RdfError(ioArgs, rdf.getElementsByTagName("rdf:Description")[0], action);
            }

            if (ioArgs.xhr.statusText) {
                var message = ioArgs.xhr.statusText + " - " + ioArgs.url;
            } else {
                var message = dojo.string.substitute(errorMessages.defaultXhrErrorMessage, [ioArgs.url]);
            }

            var details = null;
            if (ioArgs.xhr.responseText) {
                var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
                if (contentType && contentType.toLowerCase() === "text/html" ||
					ioArgs.xhr.responseText.search(/^<html>/i) != -1) {
                    var details = dojo.doc.createElement("div");
                    try {
                        details.innerHTML = ioArgs.xhr.responseText;
                    } catch (e) {
                        details = ioArgs.xhr.responseText;
                    }
                } else {
                    var details = ioArgs.xhr.responseText;
                }
            }

            var date = ioArgs.xhr.getResponseHeader("Date");
            return new carter.ui.Error({
                status: ioArgs.xhr.status || null,
                severity: this.Severity.ERROR,
                "message": message,
                detailedMessage: details,
                "date": date ? new Date(date) : null,
                action: action
            });
        }
    };

    dojo.declare("carter.ui.Error", Error, {

        status: null,
        subStatus: null,
        severity: null,
        message: null,
        detailedMessage: null, // string or dom node
        seeAlso: null,
        cause: null,
        date: null,

        action: null, // the action is a localized string which is combined with
        // a generic error message to provide context

        _isGeneric: true, //true by default              

        constructor: function(params) {
            if (params) {
                dojo.mixin(this, params);
            }
            if (!this.date) {
                this.date = new Date();
            }
            if (!this.severity) {
                this.severity = carter.ui.error.Severity.ERROR;
            }
        },

        dispatch: function() {
            dojo.publish("carter.ui.Error.event", [this._isGeneric ? this.getGenericError() : this]);
        },

        getGenericError: function() {
            // Construct a generic error message including the action
            // and set the cause of the generic error to be the real error.
            return new carter.ui.Error({
                status: this.status,
                subStatus: this.subStatus,
                severity: this.severity,
                message: (this.action ? dojo.string.substitute(errorMessages.genericError_message_action, [this.action]) : errorMessages.genericError_message),
                detailedMessage: errorMessages.genericError_detailedMessage,
                seeAlso: this.seeAlso,
                date: this.date,
                cause: this,
                _isGeneric: true // mark as generic
            });
        }
    });


    dojo.declare("carter.ui._Error", carter.ui.Error, {

        error: null,

        constructor: function(error, action) {
            this.error = error;
            this.message = error.name + ": " + error.message;
            // only add a stack trace while in debug mode
            this.detailedMessage = dojo.config.isDebug && error.stack ? error.stack : null;
            this.status = error.status != null ? error.status : error.number != null ? error.number : null;
            this.action = action
        },

        preamble: function() {
            return [];
        }

    });

    //err:Report    err:errorStatus      No.     The major status code, usually the corresponding HTTP status code.
    //err:Report    err:errorMessage     Yes.    Some textual response.
    //err:Report    err:errorSubStatus   No.     The minor status code, usually a Foundation specified value.
    //err:Report    err:errorSeverity    No.     One of the values: info, warning, error, critical.
    //err:Report    err:detailedMessage  Yes.    Any additional information the application needs to convey.
    //err:Report    err:seeAlso          No.     A URL to a resource that may contain information such as how to resolve this error.
    //err:Report    err:errorCause       No.     Another err:Report that denotes a condition that caused this error report to be generated.

    dojo.declare("carter.ui._RdfError", carter.ui.Error, {

        constructor: function(ioArgs, rdfDescription, action) {
            this.action = action;
            var date = ioArgs.xhr.getResponseHeader("Date");
            if (date) {
                this.date = new Date(date);
            }
            for (var i = 0; i < rdfDescription.childNodes.length; ++i) {
                var childNode = rdfDescription.childNodes[i];
                if (childNode.nodeType == 1 && childNode.childNodes.length > 0 && childNode.childNodes[0].nodeValue != null) {
                    switch (String(childNode.nodeName)) {
                        case 'err:errorStatus':
                            this.status = childNode.childNodes[0].nodeValue;
                            break;
                        case 'err:errorMessage':
                            this.message = childNode.childNodes[0].nodeValue;
                            break;
                        case 'err:errorSubStatus':
                            this.subStatus = childNode.childNodes[0].nodeValue;
                            break;
                        case 'err:errorSeverity':
                            this.severity = childNode.childNodes[0].nodeValue;
                            break;
                        case 'err:detailedMessage':
                            var value = childNode.childNodes[0].nodeValue;
                            if (value.search(/^<html>/i) != -1) {
                                this.detailedMessage = dojo.doc.createElement("div");
                                try {
                                    this.detailedMessage.innerHTML = value;
                                } catch (e) {
                                    this.detailedMessage = value;
                                }
                            } else {
                                this.detailedMessage = value;
                            }
                            break;
                        case 'err:seeAlso':
                            this.seeAlso = childNode.childNodes[0].nodeValue;
                            break;
                        case 'err:errorCause':
                            this.cause = new carter.ui._RdfError(ioArgs, childNode.getElementsByTagName("rdf:Description")[0]);
                            break;
                    }
                }
            }
        },

        getTagValue: function(tagName) {
			switch (tagName) {
				case 'errorStatus':
					return this.status;
				case 'errorMessage':
					return this.message;
				case 'errorSubStatus':
					return this.subStatus;
                case 'errorSeverity':
                	return this.severity;
                case 'detailedMessage':
                	return this.detailedMessage;
				case 'seeAlso':
                	return this.seeAlso;
				case 'errorCause':
					return this.cause;
            }
            return null;
        },

        preamble: function() {
            return [];
        }

    });
})();