/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
//(function() {
dojo.provide("com.ibm.rational.carter.ide.client.header.headerWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.app.ui.internal.util.UrlUtil");


dojo.requireLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");


var baseMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");


dojo.declare("com.ibm.rational.carter.ide.client.header.headerWidget", [dijit._Widget, dijit._Templated], {

	templateString: '<div>' +
				'<div dojoAttachPoint="_headerContent" class="mainContent"></div>' +
		'</div>',

	titleImg: null,

	_headerContent: null,
	
	_resource: null,
	
	_resource: this,

	constructor: function() {

	},

	postCreate: function() {

		//this.createTitleContents(true);

	},
	
	setResource: function(resource) {
		if (resource != this._resource)
		{
			this._resource = resource;
			this.createTitleContents(true);
		}
	},

	createTitleContents: function(recreate){
		if(recreate && this._headerContent){
			while(this._headerContent.hasChildNodes()) {
				var first= this._headerContent.firstChild;
		  	 	this._headerContent.removeChild(first);
		  	 	dojo.destroy(first);
			}	
		}
		this.createHeader(this._headerContent);
		if(this.editorHelp){
			this.editorHelp._helpImg.alt= baseMessages.help;			
		}
	},

	createHeader: function(parentNode) {
		if (this._resource != null)
		{
			var iconUri = this.getIconUri();
	
			if (iconUri) {
				
				var tooltip= this.getTooltip();
				
				var iconNode = dojo.create("span", {className: "title title-icon"});
				this.titleImg = dojo.create("img",{title: tooltip,alt: tooltip});
				var handles = [
						this.connect(this.titleImg, "onerror", function() {
							dojo.forEach(handles, this.disconnect, this);
							iconNode.parentNode.removeChild(iconNode);
						}),
						this.connect(this.titleImg, "onload", function() {
							dojo.forEach(handles, this.disconnect, this);
							iconNode.appendChild(this.titleImg);
						})
					];
				this.titleImg.src = iconUri;
				
				parentNode.appendChild(iconNode);
			}
		}

		this._createTitleNode(parentNode);
		
		if (this._resource != null) {
			// Optional resource number (user visible ID)
			var number = this.getResourceNumber();
			if (number != null) {
				var resourceNumberSpan = dojo.create("span", {className: "resourceNumber"});
				resourceNumberSpan.appendChild(dojo.doc.createTextNode(number + ": "));
				this.titleNode.insertBefore(resourceNumberSpan, this.titleNode.firstChild);
			}
			
			dojo.create("br", null, parentNode);
			
			var modifiedSpan = dojo.create("span", {className: "lastModified"});
			var date = dojo.date.locale.format(this.getModificationTime(), {formatLength: "medium"});
			modifiedSpan.appendChild(dojo.doc.createTextNode(
				dojo.string.substitute(baseMessages.modified, [date])
			));
			
			parentNode.appendChild(modifiedSpan);
		}
		
	},

	_createTitleNode: function(parentNode){
		this.titleNode = dojo.create("span", {
			className: "title",
			title: this.getTooltip(),
		});

		this.createTitleNode(this.titleNode, this.getTitleNodeValue(), this.getTooltip());
		this._createDirtyIndicator();
		parentNode.appendChild(this._dirtyIndicator);

		parentNode.appendChild(this.titleNode);
	},

	_createDirtyIndicator: function(){
		this._dirtyIndicator = dojo.create("span", {className: "title dirtyIndicator"});
		this._dirtyIndicator.style.display = "none";
		this._dirtyIndicator.appendChild(dojo.doc.createTextNode(" *"));
	},

	isEditableTitle: function(){
		return false;
	},
	
	getTooltip: function(){
		return this._resource ? this._resource.longName : null;
	},
	
	getTitleNodeValue: function(){
		return this._resource ? this._resource.shortName : 
			rmps.app.ui.internal.util.UrlUtil.getActionArgument("resourceName");
	},
	
	getIconUri: function() {
		return this._resource ? this._resource.iconUri : null;
	},

	getResourceNumber: function() {
		return null;
	},
	getModificationTime: function() {
		return this._resource ? this._resource.modificationTime : null;
	},	
	
	
	createTitleNode: function(titleNode, title, tooltip){
		titleNode.appendChild(dojo.doc.createTextNode(title));
		if (this.uaKey && this.uaKey != "") {
			this.editorHelp = new jazz.app.ua.PageHoverHelp({uaKey: this.uaKey});
			titleNode.appendChild(this.editorHelp.domNode);
			this.editorHelp.domNode.style.margin = "0px 0px 0px 5px";
		}
	},
	
	destroy: function(){
		this.inherited(arguments);
	},


});

//})();
