/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("com.ibm.rational.carter.ide.client.internal.RhpProjectAreaInfo");
dojo.require("rmps.core.RootServices");

dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");

	
com.ibm.rational.carter.ide.client.internal.RhpProjectAreaInfo = {
	
	_projectAreas : null,
	
	isInitialized: function() {
		return this._projectAreas != null;
	},
	
	getProjectAreas: function() {
		return this._projectAreas;
	},
	
	getProjectAreaByName: function (projectName) {
		var result = null;
		if (projectName != null)
		{
			dojo.forEach(this._projectAreas, function(data){
				var projectNameStr = new String(projectName).toUpperCase();
				var dataNameStr = new String(data.name).toUpperCase();

				if (projectNameStr == dataNameStr)
					result = data;
             }, this);
		}
		return result;
	},

	getProjectAreaById: function (projectId) {
		var result = null;
		if (projectId != null)
		{
			dojo.forEach(this._projectAreas, function(data){
				if (projectId == data.itemId)
					result = data;
             }, this);
		}
		return result;
	},
	
	getProjectAreaByUrl: function (projectUrl) {
		var result = null;
		if (projectUrl != null)
		{
			dojo.forEach(this._projectAreas, function(data){
				if (projectUrl == data.url)
					result = data;
             }, this);
		}
		return result;
	},
	
	
	init: function(){

		var ProcessEditorConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
		var PROJECT_AREA_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas");
        var projectAreasUrl = PROJECT_AREA_PATH;
		var args = {
            url: projectAreasUrl,
            headers: {
                "accept": ProcessEditorConstants.CONTENT_TYPE_APPLICATION_XML,
                "X-ibm-rmps-internal": "true"
            },
            handleAs: "xml"//, //$NON-NLS-1$
            //error: dojo.hitch(this, this._handleErrorGettingProjectAreaLinks)
        }
        var deferred = jazz.client.xhrGet(args);
		deferred.addCallback(this,this._parseProjectAreas);
		deferred.addCallback(this,this.initGroups);
		deferred.addErrback(this,function(err){
				var y = 15;
				
			});
		return deferred;
    },
	
	initGroups: function() {
		
		var projectAreas = "";
		var first = true; 
        dojo.forEach(this._projectAreas, function(projectArea){
            if (projectArea != null) {
				if (first)
					first = false;
				else
					projectAreas += ";";
				projectAreas += projectArea.itemId;
			}
        }, this);

		var ProcessEditorConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
		var GROUP_FINDER_PATH = rmps.core.RootServices.getUri("http://ibm.com/carter/1.0/", "groupfinder");
		var args = {
            url: GROUP_FINDER_PATH + "?projectAreaIds="+projectAreas,
            headers: {
                "accept": ProcessEditorConstants.CONTENT_TYPE_APPLICATION_XML,
                "X-ibm-rmps-internal": "true"
            },
            handleAs: "text"//, //$NON-NLS-1$
            //error: dojo.hitch(this, this._handleErrorGettingProjectAreaLinks)
        }
		//alert(dojo.objectToQuery(args));
        var deferred = jazz.client.xhrGet(args);
		deferred.addCallback(this,this._parseGroups);
		deferred.addErrback(this,function(err){
				var y = 15;
				alert("xmlDocument errBck");
				
			});
		return deferred;
		
	},
	_parseGroups: function(response) {
		
		if (response == null)
			return;
		var projects = response.split("$");
		for (var i = 0; i < projects.length;i++) {
			var project = projects[i];
			var data = project.split("&");
			if (data && data.length == 2) {
				var pa = this.getProjectAreaById(data[0]);
				if (pa) {
					pa.groupId = data[1];
				}
			}
		};
		//var xmlDocument = dojox.data.dom.createDocument(response); 
		//alert(dojo.objectToQuery(xmlDocument));
		//alert(xmlDocument);
		
		/*var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
        var children = xmlDocument.childNodes;
		alert("children: " + children);
        var root = UIHelper.getNode(children, "Projects"); //$NON-NLS-1$
        
        var projects = root.childNodes; //$NON-NLS-1$
        
        projects = UIHelper.getNodes(projects, "Project"); //$NON-NLS-1$
        
        for (var i = 0; i < projects.length;i++) {
			var project = projects[i];
			var projectId = project.attributes[0].value;//UIHelper.getAttributeNode(project,"projectId");//$NON-NLS-1$
			var groupId = project.attributes[1].value;//
			var pa = this.getProjectAreaById(projectId);
			if (pa)
			{
				pa.groupId = groupId; 
			}
			var y = 15;
		};*/
	},

	_parseProjectAreas: function(xmlDocument) {
		
		var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
        var children = xmlDocument.childNodes;
        var projectAreasNode = UIHelper.getNode(children, "project-areas", rmps.core.Namespaces.JP_0_6); //$NON-NLS-1$
        var projectAreaChildren = projectAreasNode.childNodes;
		var projectAreas = UIHelper.getNodes(projectAreaChildren, "project-area",  rmps.core.Namespaces.JP_0_6); //$NON-NLS-1$
        var count = projectAreas.length;
		var results = [];
        dojo.forEach(projectAreas, function(projectArea){
                var nameNode = UIHelper.getAttributeNode(projectArea,"name",rmps.core.Namespaces.JP_0_6);//$NON-NLS-1$
                
				
				var projectAreaChildren = projectArea.childNodes;

                var urlNode = UIHelper.getNode(projectAreaChildren, "url", rmps.core.Namespaces.JP_0_6); //$NON-NLS-1$
                var url = urlNode.firstChild.data;
				
				var archivedNode = UIHelper.getNode(projectAreaChildren, "archived", rmps.core.Namespaces.JP_0_6); //$NON-NLS-1$
                var archived = archivedNode.firstChild.data;
                var data = {
						itemId:url.substring(url.lastIndexOf('/')+1), //$NON-NLS-1;,
						name:nameNode.nodeValue,
						archived:archived,
						url:url
				};
				results.push(data);

            }, this);
		this._projectAreas = results;
		

	},
	
	getDefaultProjectAreas: function(resourceId,modelName) {
		
		if (resourceId == null && modelName == null && this._projectAreas == null)
			return null;
		var MODELS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");
		var templateUri = MODELS_PATH +
							"/" +
							"$projectAreaId" +
							"/" + "$groupId" +
							"/" +
							modelName +
							"/" +
							resourceId;
		var uris = [];

        dojo.forEach(this._projectAreas, function(projectArea){
                if (projectArea != null) {
					var uri = templateUri.replace("$projectAreaId", projectArea.itemId);
					uri = uri.replace("$groupId", projectArea.groupId);
					uris.push(uri);
				}
            }, this);
		
		var deferred = rmps.core.resource.ResourceInfoClient.getResourceInfo(uris);
		return deferred;
	}

	
	}
})();