/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rational.carter.ide.client.internal.RhpSubscribeButton");
dojo.require("rmps.app.ui.internal.editors.views.Links"); 

dojo.require("rmps.core.subscriptions.SubscriptionClient");

dojo.requireLocalization("rmps.ui.common", "CommonMessages");
var commonMessages = dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages");

var SubscriptionClient = rmps.core.subscriptions.SubscriptionClient;

com.ibm.rational.carter.ide.client.internal.RhpSubscribeButton = {
	
		clmWidget:null,
		_toolbar:null,
		// private properties
		_subscribed: false,
		_subscriptionToggled: false,
		_subscriptionButton: null,
		_resourceInfo:null,
		_userInfo:null,
	
	init: function(toolbar,resourceInfo, userInfo, clmWidget) {
			this.clmWidget = clmWidget,
			this._toolbar = toolbar;
			this._resourceInfo = resourceInfo;
			this._userInfo = userInfo;
			this._subscriptionButton = this._toolbar.addButton(this, this._toggleSubscription, {
				label: "subscribe",
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/subscribe_co.gif")
			});
			this._toolbar.hideItem(this._subscriptionButton);
			if (this._resourceInfo != null && this._userInfo != null) {
			
				var deferred = SubscriptionClient.getSubscriptions(this._resourceInfo.uri, this._userInfo.uri);
				deferred.addCallback(this, this._handleSubscriptions);
				deferred.addErrback(this, function(error){
					if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
						rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
					}
					else {
						rmps.core.error.createError(error).dispatch();
					}
				});
			}

		},
	update: function(toolbar,resourceInfo, userInfo, clmWidget) {
		//reset subscription toggled.
		this._subscriptionToggled = false;
		if (this._subscriptionButton == null || this._toolbar == null)
		{
			this.init(toolbar,resourceInfo, userInfo, clmWidget);
			return;
		}
		this._resourceInfo = resourceInfo;
		this._userInfo = userInfo;
		this._toolbar.hideItem(this._subscriptionButton);
		if (this._resourceInfo != null && this._userInfo != null) {
			
				var deferred = SubscriptionClient.getSubscriptions(this._resourceInfo.uri, this._userInfo.uri);
				deferred.addCallback(this, this._handleSubscriptions);
				deferred.addErrback(this, function(error){
					if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
						rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
					}
					else {
						rmps.core.error.createError(error).dispatch();
					}
				});
			}

		},	
	_handleSubscriptions: function(subscriptions) {
		// If any subscriptions are retrieved then the user is already subscribed to this resource
		this._subscribed = subscriptions && subscriptions.length > 0;
		
		this._updateSubscriptionButton();
		this._toolbar.showItem(this._subscriptionButton);
	},
	
	_toggleSubscription: function() {
		this._subscriptionToggled = !this._subscriptionToggled;
		this._updateSubscriptionButton();
		if (this._subscriptionToggled) {
			this.clmWidget.addDirty("subscriptionToggled");
		} else {
			this.clmWidget.removeDirty("subscriptionToggled");
		}
	},
	
	isSubscribed: function() {
		return (this._subscribed || this._subscriptionToggled) && !(this._subscribed && this._subscriptionToggled);
	},
	
	subscribe: function() {
		if (!this._subscribed && !this._subscriptionToggled) {
			this._toggleSubscription();
		}
	},
	
	unsubscribe: function() {
		if (!this._subscribed && this._subscriptionToggled) {
			this._toggleSubscription();
		}
	},
	
	_updateSubscriptionButton: function() {
		
		// if subscribed, show the unsubscribe icon
		if (this.isSubscribed()) {
			this._toolbar.updateButton(this._subscriptionButton, {
				label: "messages.unsubscribe",
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/unsubscribe_co.gif")
			});
		// else unsubscribed, show the subscribe icon
		} else {
			this._toolbar.updateButton(this._subscriptionButton, {
				label: "messages.subscribe",
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/subscribe_co.gif")
			});
		}
	},
	
	/*
	 * @see rmps.core.TaskMonitor
	 */
	doSave: function(taskMonitor) {

		// save subscription if toggled
		if (this._subscriptionToggled) {
			if (this._subscribed) {
				var task = taskMonitor.beginTask("Unsubscribe", 1);
				var deferred = SubscriptionClient.unsubscribe(this._resourceInfo.uri, this._userInfo.uri);
				deferred.addCallback(this, function(result) {
					this.clmWidget.removeDirty("subscriptionToggled");
					this._subscribed = false;
					this._subscriptionToggled = false;
				});
				task.handleDeferredWork(deferred, 1);
			} else {
				var task = taskMonitor.beginTask("Subscribe", 1);
				var deferred = SubscriptionClient.subscribe(this._resourceInfo.uri, this._userInfo.uri);
				deferred.addCallback(this, function(result) {
					this.clmWidget.removeDirty("subscriptionToggled");
					this._subscribed = true;
					this._subscriptionToggled = false;
				});
				task.handleDeferredWork(deferred, 1);
			}
			
		}
	}
	
}
