/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2010.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/
// declare the Dojo module
dojo.provide("carter.ui.publisher.ManageModelsEditor");

// Dojo requirements
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.parser");
dojo.require("dojo.io.iframe");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo._base.xhr");
dojo.require("dojo.i18n");

//jazz/kenton requirements
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.core.RootServices");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.ui.EditableCollapsibleSection");

// Carter reuqirements: 
dojo.require("carter.ui.publisher.ManageModels");
dojo.require("carter.ui.publisher.Model");
dojo.require("carter.ui.error");
dojo.require("carter.ui.utils");

dojo.requireLocalization("carter.ui", "ManageModelMessages");
var messages = dojo.i18n.getLocalization("carter.ui", "ManageModelMessages");

var Utils = carter.ui.utils;

//Context help IDs:
var UAKEY_MANAGE_MODELS_PAGE = "com.ibm.rational.carter.web.page_ManageModelsEditor001";
var UAKEY_MODEL_ROW = "com.ibm.rational.carter.web.text_ManageModelsEditor001";

// declare and implement the widget "class"
dojo.declare("carter.ui.publisher.ManageModelsEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "carter-ui-publisher-ManageModelsEditor",
	
	// private properties
	modelsList: null,

	constructor: function() {
		// set context help id to the editor
		this.uaKey = UAKEY_MANAGE_MODELS_PAGE;
	},

    postCreate: function() {
        this.inherited(arguments);
        this._initialize();        
    },

	_initialize: function() {

		if( this.modelsList == null ){
			this.modelsList = new carter.ui.publisher.ManageModels._list({editor: this});
			this.containerNode.appendChild(this.modelsList.domNode);
			this.modelsList.startup();							
		}
	},

    startup: function(){
		this.hideMsg();
	},
	
	addButtons: function(toolbar) {		
		this.inherited(arguments);
		
		// hide the default save button
		this.saveButton.domNode.style.display = "none";
	},
	
	refresh: function() {
		if( this.modelsList != null )
			this.modelsList.loadContent();
	},
	
	showInfoMsg: function(msg){
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: msg,
			closable: true
		});
	},
	
	showOkMsg: function(msg){
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: msg,
			closable: true
		});
	},

	showErrorMsg: function(msg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: msg,
			closable: true
		});
	},

	hideMsg: function(){
		this.hideMessage();
	}
});

// Widget implementation:
dojo.declare("carter.ui.publisher.ManageModels._list", [dijit._Widget, dijit._Templated], {

	messages: null,
	editor: null,
	_table: null,
	_deferred: null,
	_updateTimer: null,
	_deferredDeletion: null,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
		'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="false""></div>' +
		'</div>',

	constructor: function(arguments){
		this.editor =  arguments.editor;
		this.messages = messages;
	},
	
	postCreate: function() {
		this.inherited(arguments);

		// Create the table:
		this._table = new jazz.ui.StyledTable({
		topBorder: true,
		columns: [
			{
				label: this.messages.modelName,
				width: "50%"
			},
			{
				label: this.messages.description,
				width: "50%"
			},
			{
				label: this.messages.actions,
				className: "actions",
				width: "20%"
			} ]	
		});

		this._section.headerNode.style.display = "none";
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();

		this.loadContent();
		
		// start update interval: 
		if( ! this._updateTimer ){
			this._updateTimer = setInterval(dojo.hitch(this, this.loadContent), 600000); // 10 minutes
		}
		
	},
	
	destroy: function() {
   		if (this._updateTimer) {
			clearInterval(this._updateTimer);
			delete this._updateTimer;
		} 	
		if (this._deferred) {
			this._deferred.cancel();
			delete this._deferred;
		}
        this.inherited(arguments);
    },
	
	loadContent: function() {
	
		// pervent concurrent requests:
		if (this._deferred) return;
		if( this._table == null ) return;
		
		this.resetContent();

		var ModelsMgr = carter.ui.publisher.ManageModels;
		if( ! ModelsMgr ){
			alert(messages.err_NoPublishObj);
			return;
		}
		
		if( this.editor){
			this.editor.hideMsg();
		}
		
		// Populate the table:
		var projectId = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId;
		this._deferred = ModelsMgr.getStoredModels(projectId);
		
		if( this._deferred != null ){
			this._deferred.addCallback(this, function(models) {
				function ModelSort(a, b) {
					var aLower = a.name.toLowerCase();
					var bLower = b.name.toLowerCase();
					return aLower < bLower ? -1 : 1;
				}

				// need to sort by model names
				//models.sort(ModelSort);
				for( var i = 0 ; i < models.length; ++i) {
					var row = new carter.ui.publisher.ManageModels._Row({list: this});
					this._table.tbody.appendChild(row.domNode);
					row.setParams(models[i].name, "", 1);
				}
			});			
			this._deferred.addErrback(this, function(error) {
				var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			this._deferred.addBoth(this, function() {
				delete this._deferred;
			});	
		}		
	},
	
	updateListButtons: function(){
		this._section._previewButton.style.display = "none";
		this._section._editButton.style.display = "none";			
	},
	
	scheduleDeletion: function(row){

		var ModelsMgr = carter.ui.publisher.ManageModels;
		if( ! ModelsMgr ){
			alert(this.messages.err_NoPublishObj);
			return;
		}

		var projectId = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId;
		this._deferredDeletion = ModelsMgr.deleteModel(row._modelName, projectId);
		
		if( this._deferredDeletion != null ){
			this._deferredDeletion.addCallback(this, function(retVal) {
				if( this.editor){
					var msg = dojo.string.substitute(messages.inf_DelScheduledOk, [Utils.getActivitiesEditorUri() + "&refresh=true"]);
					this.editor.showOkMsg(msg);
				}
			});
			this._deferredDeletion.addErrback(this, function(error) {
				var errMsg = carter.ui.error.getCarterError(this._deferredDeletion.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrorMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._deferredDeletion.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferredDeletion.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			this._deferredDeletion.addBoth(this, function() {
				delete this._deferredDeletion;
			});	
		}
	},

	resetContent: function() {
		var count = 0;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			row.destroyRecursive();
			delete row;
			count += 1;
		});
		
		dojox.xml.parser.removeChildren(this._table.tbody);
	}
});

dojo.declare("carter.ui.publisher.ManageModels._Row", dijit._Widget, {
	
	_modelName: null,
	_description: null,
	
	_modelNameTd: null,
	_descriptionTd: null,

	_deleteModelDeferred: null,
	
	_popupContent: null,
	_popup: null,
	_actionBar: null,
	_deleteAction: null,
	list: null,
	
	messages: null,
	
	constructor: function(arguments){
		this.list =  arguments.list;
		this.messages = messages;
	},
	
	buildRendering: function() {
		this.domNode = this.containerNode = dojo.create("tr");
	},
	
	postCreate: function() {
		// name
		this._modelNameTd = dojo.create("td", null, this.domNode);
		this._modelNameTd.className = "name";
		this._updateModelName();
		
		this._popup = new jazz.ui.HoverPopup({
			around: this._modelNameTd,
			onOpen: dojo.hitch(this, this._createPopupContents),
			maxWidth: 800,
			maxHeight: dojo.isIE ? 570 : 550
		});
		
		// description
		this._descriptionTd = dojo.create("td", null, this.domNode);
		this._descriptionTd.className = "description";
		this._updateDescription();
						
		// actions
		var actionsTd = dojo.create("td", null, this.domNode);
		this._actionBar = new jazz.ui.SimpleToolbar();
		actionsTd.appendChild(this._actionBar.domNode);
		this._actionBar.startup();
		this._updateActions();		
	},
	
	setParams: function(name, description){
		this._modelName = name;
		this._description = description;
		
		this._updateModelName();
		this._updateDescription();
	},
	
	_deleteModel: function() {
		this._actionBar.disableButton(this._deleteAction);
		if( this.list != null ){
			this.list.scheduleDeletion(this);
		}
	},
	
	_updateActions: function() {
		this._deleteAction = this._actionBar.addButton(this, this._deleteModel, {
			label: this.messages.schedule_deletion,
			imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/remove.gif")
		});			
	},
	
	_updateModelName: function() {
		if( this._modelNameTd != null ) {
			this._modelNameTd.innerHTML = "";
			if( this._modelName != null ) {
				this._modelNameTd.appendChild(dojo.doc.createTextNode(this._modelName));
			}
		}		
	},
	
	_updateDescription: function() {
		if( this._descriptionTd != null ) {
			this._descriptionTd.innerHTML = "";
			if( this._description != null ) {
				this._descriptionTd.appendChild(dojo.doc.createTextNode(this._description));
			}
		}		
	},
	
	_createPopupContents: function() {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
		}
	}
});
