/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2010.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/
// declare the Dojo module
dojo.provide("carter.ui.publisher.NewPublishEditor");

// dojo includes:
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.parser");
dojo.require("dojo.io.iframe");
dojo.require("dojo._base.xhr");

// Jazz & rmps includes:
dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.FilterBox");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.core.RootServices");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("jazz.ui.StyledTable");

// Carter includes:
dojo.require("carter.ui.publisher.Publisher");
dojo.require("dojo.i18n");
dojo.requireLocalization("carter.ui", "PublisherMessages");
dojo.require("carter.ui.utils");
dojo.require("carter.ui.error");

var messages = dojo.i18n.getLocalization("carter.ui", "PublisherMessages");

var Utils = carter.ui.utils;

// Context help IDs:
var UAKEY_PUBLISH_PAGE = "com.ibm.rational.carter.web.page_NewPublishEditor01";

// declare and implement the widget "class"
dojo.declare("carter.ui.publisher.NewPublishEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "carter-ui-publisher-newPublishEditor",
	
	widget: null,
	
	constructor: function() {
		// set context help id to the editor
		this.uaKey = UAKEY_PUBLISH_PAGE;
	},

    postCreate: function() {
        this.inherited(arguments);
        this._init();        
    },
    
    startup: function(){
    	this.hideMessage();
    	if( this.widget != null ){
    		this.widget.resetContent();
    	}
    },
    
    _init: function(definition) {		
		this._createSections();
	},
	
	_createSections: function() {
		this._createDecorator();
		this._createMainSection();
	},

	_createDecorator: function() {
		var seperator = new carter.ui.publisher.NewPublishEditor._Decorator();
		this.containerNode.appendChild(seperator.domNode);
		seperator.startup();
	},

	_createMainSection: function() {
		this.widget = new carter.ui.publisher.NewPublishEditor._General({editor: this});
		this.containerNode.appendChild(this.widget.domNode);
		this.widget.startup();		
	},
	
	addButtons: function(toolbar) {		
		this.inherited(arguments);
		
		// hide page buttons (top-right corner) 
		this.saveButton.domNode.style.display = "none";
		this.refreshButton.domNode.style.display = "none";
	},
	
	showInfoMsg: function(msg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: msg,
			closable: true
		});		
	},
	showErrMsg: function(msg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: msg,
			closable: true
		});
	},
	showOkMsg: function(msg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: msg,
			closable: true
		});		
	},
	
	hideMsg: function(){
		this.hideMessage();
	}
});

dojo.declare("carter.ui.publisher.NewPublishEditor._Decorator", [dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	_table: null,
	
	templateString: '<div>' +
		'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="false""></div>' +
		'</div>',
	
    postCreate: function() {
		this.inherited(arguments);		
		this._section.headerNode.style.display = "none";
	},
});

dojo.declare("carter.ui.publisher.NewPublishEditor._General", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("carter.ui", "internal/templates/PublishWidget.html"),
	widgetsInTemplate: true,
	editor: null,
	messages: null,
	_validateDeferred: null,
	
	constructor: function(arguments){
		this.messages = messages;
		this.editor = arguments.editor;	
	},
		
    postCreate: function() {
		this.inherited(arguments);
		
		// NLS support:
		if( this.publishBtn != null )
			this.publishBtn.value = this.messages.publish;
		if( this.modelFileTitle != null)
			this.modelFileTitle.innerHTML = this.messages.zippedModel;
		
		// disable the submit button till valid selection is made:
		this.publishBtn.disabled = true;
	},

	doPublish: function() {
		var Publisher = carter.ui.publisher.Publisher;
		if( ! Publisher ){
			alert(messages.err_NoPublishObj);
			return;
		}

		if( this.editor ){
			this.editor.showInfoMsg(this.messages.publishing);
		}
	
		// block consecutive calls:
		this.publishBtn.disabled = true;
		
		// launch model server validation request:
		var projectId = jazz.app.context.get().itemId;
		this._validateDeferred = Publisher.validatepublishModel(projectId);
		
		if( this._validateDeferred != null ){
			this._validateDeferred.addCallback(this, function() {
				// Model Serve was validated. Launch the import request:
				var projectId = jazz.app.context.get().itemId;
				var Publisher = carter.ui.publisher.Publisher;
				var errCode = Publisher.publishZippedModel(projectId);
				if( errCode != null && this.editor != null){
					var errDescript = Utils.getErrorDescription(errCode);
					if( ! errDescript ){
						errDescript = errCode;
					}
					this.editor.showErrMsg(errDescript);
				}
				else if (this.editor != null){
					var msg = dojo.string.substitute(this.messages.inf_ImportScheduledOk, [Utils.getActivitiesEditorUri() + "&refresh=true"]);
					this.editor.showOkMsg(msg);
				}
			});
			this._validateDeferred.addErrback(this, function() {
				// Model server validation failed:
				var errMsg = carter.ui.error.getCarterError(this._validateDeferred.ioArgs);
				if( errMsg != null && this.editor != null){
					this.editor.showErrMsg(errMsg);
					return;
				}
				if (!dojo._isDocumentOk(this._validateDeferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._validateDeferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			this._validateDeferred.addBoth(this, function() {
				delete this._validateDeferred;
				this.publishBtn.disabled = false;
			});	
		}
	},
	
	_onSelectionChanged: function(){
		var filename = this.modelFile.value;
		var temp = filename.substr(filename.length - 3, 3);
		var extension = temp.toLowerCase();
		if( extension == "zip"){
			this.publishBtn.disabled = false;			
		}
		else {
			this.publishBtn.disabled = true;
		} 
	},
	
	resetContent: function(){
		this.modelFile.value = "";		
	}
});
