/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rational.carter.ide.client.search.RelatedElementsWidget");

/*######################################### */
/*These imports are required to invoke auth dialogs, and
 * users' credentials checking utils. Otherwise, the users,
 * will not be autheroized when trying to search for an element
 * even if they are already logged in */
dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
dojo.require("jazz.app.proxy");

dojo.require("rmps.app.ui.internal.editors.views.RelatedElements");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("dojo.hash");

dojo.require("com.ibm.rational.carter.ide.client.resource.RhpResourceInfoClient");
dojo.require("com.ibm.rational.carter.ide.client.internal.RhpProjectAreaInfo");
dojo.require("com.ibm.rational.carter.ide.client.internal.RhpCommunicator");

var ResourceInfoClient = com.ibm.rational.carter.ide.client.resource.RhpResourceInfoClient;
var RhpProjectAreaInfo = com.ibm.rational.carter.ide.client.internal.RhpProjectAreaInfo;
var RhpCommunicator = com.ibm.rational.carter.ide.client.internal.RhpCommunicator;

dojo.requireLocalization("rmps.app.ui.internal.editors.views", "RelatedElementsMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views", "RelatedElementsMessages");


dojo.declare("com.ibm.rational.carter.ide.client.search.RelatedElementsWidget", rmps.app.ui.internal.editors.views.RelatedElements, {
	
	tableClass: "rmps.ui._Table",
	_resource:null,
	_label:null,
	_projectArea: null,
	
	postCreate: function() {
		
		var deferred = RhpProjectAreaInfo.init();
		deferred.addCallback(this,this.initResource);

		//subscribe to show in browser event.		
		 dojo.subscribe("rmps.web.app.explorer.show", this, function(resourceUri){
		 	if (resourceUri != null) {
				RhpCommunicator.notifyRhapsody({
					cmd: "ShowInExplorer",
					params:[resourceUri.uri]
				});
				//var location = "Search:$ShowInExplorer$rhp sep" + resourceUri.uri;
				//this.setWindowLocation(location);
			}
 		});
		
		
		net.jazz.ajax.ui.PlatformUI._workbench = this;
		this.inherited(arguments);
		
		//subscribe on has change to catch pageSize changes
		dojo.subscribe("/dojo/hashchange", this, "onHashChange");
	},
	
	createTable: function() {
		var table= this.inherited(arguments);
	//	table._projectArea= this._projectArea;
		return table;
	},

	initResource: function() {
		
		var uri = this.getFragment("resourceUri");
		var deferred = ResourceInfoClient.getResourceInfo(uri);
		deferred.addCallback(this, function(infos){
			this._resource = (infos != null)? infos[0]:null;
			if (this._resultTable) {
				this._resultTable._projectArea = RhpProjectAreaInfo.getProjectAreaByUrl(this._resource.resourceContext);
				
			}
			if (this._label != null) {
				var content = "Related Elements of: " + this._resource.shortName;
				
				this._label.innerHTML = content;
				
			}

		}); 
		deferred.addCallback(this,this.initOptions);
	},
	onHashChange: function(hash) {
		this.initResource();
		if (this._blankOption == null || this._selectBox.value != this._blankOption.value) 
			this._onQueryChange();
	},
	
	initOptions: function(){
		
		
		var hasBlankOption = this._blankOption != null;
		while (this._selectBox.hasChildNodes())
		{
			var opt = this._selectBox.removeChild(this._selectBox.lastChild);
			delete opt;
		}
		if (hasBlankOption) 
			this._blankOption = this.addOption(messages.selectRelationship);
		if (this._resource) {
			if (this._resource.diagramName != null){
				this.addOption(this.Queries.REFERENCED_ELEMENTS_OF_DIAGRAM.name, this.Queries.REFERENCED_ELEMENTS_OF_DIAGRAM.value);
				this.addOption(this.Queries.REFERENCED_DIAGRAMS_OF_DIAGRAM.name, this.Queries.REFERENCED_DIAGRAMS_OF_DIAGRAM.value);
				this.addOption(this.Queries.REFERENCING_DIAGRAMS_OF_ELEMENT.name, this.Queries.REFERENCING_DIAGRAMS_OF_ELEMENT.value);
			} 
			else {
				this.addOption(this.Queries.REFERENCED_ELEMENTS_OF_ELEMENT.name, this.Queries.REFERENCED_ELEMENTS_OF_ELEMENT.value);
				this.addOption(this.Queries.REFERENCING_ELEMENTS_OF_ELEMENT.name, this.Queries.REFERENCING_ELEMENTS_OF_ELEMENT.value);
				this.addOption(this.Queries.REFERENCING_DIAGRAMS_OF_ELEMENT.name, this.Queries.REFERENCING_DIAGRAMS_OF_ELEMENT.value);
			}
		}
	},
	
	
	
	getFragment: function(attr) {
		var hash = dojo.hash();
		var fragment = dojo.queryToObject(hash);
		return fragment[attr];
	},
	
	createHeader: function(parentNode) {
		
		this.inherited(arguments);
		this._label = dojo.doc.createElement("span");
		this._label.className = "header_label";
		this._label.style.width = "100%";
		this._label.style.align = "center";
		this._label.appendChild(dojo.doc.createTextNode(""));
		parentNode.appendChild(this._label);
		
	},
	
	onQueryChange: function(queryId) {
		var query = new rmps.app.core.internal.query.RelationshipTextQuery();
		query.relationship = queryId;
		query.uri = this.getFragment("resourceUri");
		query.pageSize = 50;
		query.orderBy = [{id: "modified", isAscending: false}, {id: "name", isAscending: false}];
		query.context = (this._resource!=null)?this._resource.resourceContext:null;//[ dojo.getObject("jazz.app.currentApplication").context ];
		query.name = this.Queries.get(queryId).name;
		query.baseline = null;//jazz.app.context.get().baseline;
		this.executeQuery(query);
	},
	
	showMessage: function(args, closeDelay){
	
	},
	hideMessageArea: function() {
		
	},

});

	dojo.declare("rmps.ui._Table",rmps.ui.Table, {
		
	_baseLinkUri: null,
	_projectArea: null,
	
	_populateTableBody: function() {
		
		if (this._projectArea != null)
		{
			var PROJECT_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "projects");
			PROJECT_PATH = PROJECT_PATH.replace("projects","web/projects");
			this._baseLinkUri = PROJECT_PATH + "/" + this._projectArea.name;
		}
		this.inherited(arguments);
	},

	_populateColumn: function(rowConfig, tableRow, rowIdx, columnConfig, tableCell, colIdx){
		this.inherited(arguments);
		if (columnConfig.link) {
			var span = tableCell.lastChild;
			var link = span.lastChild;
			var href = new String(link.href);
			href = href.substring(href.lastIndexOf('#'), href.length);
			link.href = this._baseLinkUri + (href);
		}
	}
		});


})();

