/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rational.carter.ide.client.search.SearchController");

/*######################################### */
/*These imports are required to invoke auth dialogs, and
 * users' credentials checking utils. Otherwise, the users,
 * will not be autheroized when trying to search for an element
 * even if they are already logged in */
dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
dojo.require("jazz.app.proxy");
/*######################################### */


dojo.require("rmps.core.query.QueryClient");
dojo.require("rmps.core.error");

dojo.require("rmps.app.core.internal.query.TextQuery");
dojo.require("rmps.app.core.internal.query.ClientPagedQueryResult");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.app.pickers", "RMPSPickerMessages");

var QueryClient = rmps.core.query.QueryClient;

dojo.declare("com.ibm.rational.carter.ide.client.search.SearchController", null, {

	messages: dojo.i18n.getLocalization("rmps.app.pickers", "RMPSPickerMessages"),
	
	// private properties
	_pickerWidget: null,
	
	_prevQueryResult: null,
	_searchDeferred: null,
	
	constructor: function(widget) {
		this._pickerWidget = widget;
	},
	
	search: function(args) {
		if (!args || !args.query) {
			return;
		}
		
		var timestamp = args.ts;
		var queryParams = dojo.queryToObject(args.query);
		
		var query = new rmps.app.core.internal.query.TextQuery_(queryParams);
		query.timestamp = timestamp;
		query.name = dojo.string.substitute(this.messages.statusSearchMessage, [query.name]);
		
		/*Inform the user that the search process is in progress */
		this._pickerWidget.showStatusNode(query.name);
		 
		if (timestamp == null) {
			timestamp = new Date().getTime();
		}
		if (this._prevQueryResult && query.equals(this._prevQueryResult.query) &&
				(this._prevQueryResult.query.timestamp == query.timestamp || query.timestamp == null)) {
			this._pickerWidget.showSearchResults(this._prevQueryResult, true);
			/*the search is done, then, remove the status message */
			this._pickerWidget.hideStatusNode(query.name);
			
		} else {
			var preserveFilter = true;
			if (!this._prevQueryResult || !query.token) {
				preserveFilter = false;
			}
			
			if (this._prevQueryResult && this._prevQueryResult.id == query.token) {
				this._prevQueryResult = new rmps.app.core.internal.query.ClientPagedQueryResult({
					query: query,
					id: this._prevQueryResult.id,
					rawEntries: this._prevQueryResult.rawEntries,
					totalResults: this._prevQueryResult.totalResults,
					uri: this._prevQueryResult.uri
				});
				this._pickerWidget.showSearchResults(this._prevQueryResult, preserveFilter);
				
			} else {
				var queryToExecute = dojo.clone(query);
				delete queryToExecute.page;
				delete queryToExecute.pageSize;
				delete queryToExecute.orderBy;
				delete queryToExecute.token;
				
				if (this._searchDeferred) {
					this._searchDeferred.cancel();
					delete this._searchDeferred;
				}
				
				this._searchDeferred = QueryClient.execute(queryToExecute);
				
				this._searchDeferred.addCallback(this, function(queryResult) {
					this._prevQueryResult = new rmps.app.core.internal.query.ClientPagedQueryResult({
						query: query,
						id: queryResult.id,
						rawEntries: queryResult.entries,
						totalResults: queryResult.totalResults,
						uri: queryResult.uri	
					});
					
					this._pickerWidget.showSearchResults(this._prevQueryResult, preserveFilter);
				});
				
				this._searchDeferred.addErrback(this, function(error) {
					if (error.dojoType != "cancel") {
						if (!dojo._isDocumentOk(this._searchDeferred.ioArgs.xhr)) {
							rmps.core.error.createXhrError(this._searchDeferred.ioArgs).dispatch();
						} else {
							rmps.core.error.createError(error).dispatch();
						}
					}
				});
			}
			if (this._searchDeferred) {
				this._searchDeferred.addBoth(this, function(){
					delete this._searchDeferred;
					
					/*the search is done, then, remove the status message */
					this._pickerWidget.hideStatusNode(query.name);
				});
			}
		}
	}
});

dojo.declare("rmps.app.core.internal.query.TextQuery_", rmps.app.core.internal.query.TextQuery, {
	
	equals: function(obj) {
		return this.inherited(arguments) && this.projectId == obj.projectId;
	}
});

})();
