/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/
(function() {
    dojo.provide("carter.ui.utils");

	dojo.require("dojo.i18n");
	dojo.requireLocalization("carter.ui", "ErrorMessages");
	var messages = dojo.i18n.getLocalization("carter.ui", "ErrorMessages");

    carter.ui.utils = {

        process: {

            Operations: {
    			IMPORTER: "com.ibm.xtools.rmps.process.operations.importer"
            },

            Actions: {
                CREATE: "create",
                DELETE: "delete",
                MODIFY: "modify",
                REQUEST_VALIDATE: "request/validate",
                REQUEST_NO_VALIDATE: "request/novalidate"
            }
        },

        _basicPrefix: "#action=carter.ui.",

		// http requests:
 		REQUEST_ENUM_DEFINITIONS: "storedDefintions",
 		REQUEST_DELETE_DEFINITION: "deleteDefinition",
 		REQUEST_SAVE_DEFINITION: "saveDefinition",
 		REQUEST_GET_DEFINITION: "getDefinition",
 		REQUEST_EXECUTE_DEFINITION: "executeDefinition",
 		REQUEST_ENUM_MODELS: "storedModels",
 		REQUEST_DELETE_MODEL: "deleteModel",

		// http respond parameters:
		PARAM_MODELS: "DefinitionModels",
		PARAM_SCHEDULEENABLE: "scheduleEnable",
		PARAM_SCHEDULETYPE: "scheduleType",
		PARAM_SCHEDULEINTERVAL: "scheduleInterval",
		PARAM_SCHEDULETIME: "scheduleTime",
		PARAM_SCHEDULEDAYS: "scheduleDays",
		PARAM_ERRORCODE: "errorCode",
		PARAM_ETAG: "ETag",
		PARAM_RETCODE: "returnCode",
		PARAM_DEFID_SHORT: "definitionId",
		
		// http request parameters:
		PARAM_PROJID: "projectId=",
		PARAM_DEFID: "definitionId=",
		PARAM_REQUESTID: "requestId=",
		PARAM_ITEMSLIMIT: "itemsLimit=",
		
		// http request types:
		REQTYPE_EXECUTEDEF: "executeDefinition",
		REQTYPE_DEFSTATUS: "definitionStatus",
		
		// error codes:
		ERR_DEFNAME_USED: 			"DefNameAlreadyInUse",
		ERR_MODELNAME_USED: 		"ModelNameAlreadyInUse",
		ERR_INSUFFICIENTPRIVILEGES: "InsufficientPrivileges",
		
		// return coded:
		RETVAL_COMPLETEDOK:			"completedOk",
		RETVAL_REQUESTFAILED:		"requestFailed",
		
		// returned error codes:
		ERRCODE_DEFNAMEALREADYUSED: 	"RHAPERR:defNameAlreadyUsed",
		ERRCODE_MODELNAMEALREADYUSED: 	"RHAPERR:modelNameAlreadyUsed",
		ERRCODE_DEFNOTEXIT: 			"RHAPERR:defNotExists",
		ERRCODE_INVALIDDEFID: 			"RHAPERR:invalidDefId",
		ERRCODE_INVALIDDEFNAME: 		"RHAPERR:invalidDefName",
		ERRCODE_UNKNOWNSCHEDULETYPE: 	"RHAPERR:unknownScheduleType",
		ERRCODE_INVALIDSCHEDULETYPE: 	"RHAPERR:invalidScheduleType",
		ERRCODE_INVALIDSCHEDULEINTERVAL: "RHAPERR:invalidScheduleInterval",
		ERRCODE_INVALIDTIMEFORMAT: 		"RHAPERR:invalidTimeFormat",
		ERRCODE_MISSINGSCHEDULETIME: 	"RHAPERR:missingScheduleTime",
		ERRCODE_INVALIDWEEKDAY: 		"RHAPERR:invalidWeekDay",
		ERRCODE_DELDEFFAILED: 			"RHAPERR:deLDefFailed",
		ERRCODE_MISSINGPARAMS:			"RHAPERR:missingParams",
		ERRCODE_DEFISMODIFIEDBYOTHER:	"RHAPERR:defIsModifiedByOther",
		ERRCODE_INTERNALSERVERR:		"RHAPERR:internalServErr",
		ERRCODE_LAUNCHMODELSERVFAILED:	"RHAPERR:failedLaunchRhapModelServer",
		ERRCODE_INVALIDPERMISSIONS:		"RHAPERR:invalidPermissions",
		ERRCODE_MODELSERVERNOTCONFIGURED:	"RHAPERR:missingModelServConfig",
		
		// Activity states:
		ACTIVITY_STATE_ERROR:					"error",
		ACTIVITY_STATE_COMPLETED:				"completed",
		ACTIVITY_STATE_IN_PROGRESS:				"inProgress",
		ACTIVITY_STATE_PENDING:					"pending",
		ACTIVITY_STATE_DROPPED:					"dropped",
		ACTIVITY_STATE_CANCELING:				"canceling",
		ACTIVITY_STATE_CANCELED:				"canceled",
		ACTIVITY_STATE_COMPLETE_WITH_WARNING:	"warning",
		ACTIVITY_STATE_UNKNOWN:					"unknown",
		
		// Activity types:
		ACTIVITY_TYPE_MODELDELETION:		"modelremoved",
		ACTIVITY_TYPE_MODELIMPORT:			"modelimport",
		ACTIVITY_TYPE_DEFINITIONIMPORT:		"definitionimport",
		
		
		// Timer constants:
		TIMEINTERVAL_ACTIVITIES_REFRESH:		600000, // 10 min.
		TIMEINTERVAL_DEFINITIONS_REFRESH:		600000, // 10 min.
		TIMEINTERVAL_ACTIVITY_PROGRESS:			10000, // 10 sec.
		TIMEINTERVAL_DEF_STATUS:				300000, // 5 min.
		
		// Definition status codes:
		DEFSTAT_ACTIVE:							"active",
		DEFSTAT_INACTIVE:						"inactive",
		
        getNewPublishViewUri: function() {
            return this._basicPrefix + "publish.viewNewPublish";
        },

        getManageModelsViewUri: function() {
			return this._basicPrefix + "publish.viewManageModels";
		},
		
		getDefinitionsViewUri: function() {
			return this._basicPrefix + "publish.viewDefinitions";
		},	

		getDefinitionEditorUri: function() {
			return this._basicPrefix + "publish.viewDefinitionEditor";
		},
		
		getActivitiesEditorUri: function() {
			return this._basicPrefix + "publish.viewActivities";
		},	

		getErrorDescription: function(errCode) {
			if( errCode == carter.ui.utils.ERR_DEFNAME_USED ) 				return messages.errMsg_DEFNAMEALREADYUSED;
			if( errCode == carter.ui.utils.ERR_MODELNAME_USED) 				return messages.errMsg_MODELNAMEALREADYUSED;
			if( errCode == carter.ui.utils.ERR_INSUFFICIENTPRIVILEGES)		return messages.errMsg_INSUFFICIENTPRIVILEGES;
			if( errCode == carter.ui.utils.ERRCODE_DEFNAMEALREADYUSED)		return messages.errMsg_DEFNAMEALREADYUSED;
			if( errCode == carter.ui.utils.ERRCODE_MODELNAMEALREADYUSED)	return messages.errMsg_MODELNAMEALREADYUSED;
			if( errCode == carter.ui.utils.ERRCODE_DEFNOTEXIT)				return messages.errMsg_DEFNOTEXIT;
			if( errCode == carter.ui.utils.ERRCODE_INVALIDDEFID)			return messages.errMsg_INVALIDDEFID;
			if( errCode == carter.ui.utils.ERRCODE_INVALIDDEFNAME)			return messages.errMsg_INVALIDDEFNAME;
			if( errCode == carter.ui.utils.ERRCODE_UNKNOWNSCHEDULETYPE)		return messages.errMsg_UNKNOWNSCHEDULETYPE;
			if( errCode == carter.ui.utils.ERRCODE_INVALIDSCHEDULETYPE)		return messages.errMsg_INVALIDSCHEDULETYPE;
			if( errCode == carter.ui.utils.ERRCODE_INVALIDSCHEDULEINTERVAL)	return messages.errMsg_INVALIDSCHEDULEINTERVAL;
			if( errCode == carter.ui.utils.ERRCODE_INVALIDTIMEFORMAT)		return messages.errMsg_INVALIDTIMEFORMAT;
			if( errCode == carter.ui.utils.ERRCODE_MISSINGSCHEDULETIME)		return messages.errMsg_MISSINGSCHEDULETIME;
			if( errCode == carter.ui.utils.ERRCODE_INVALIDWEEKDAY)			return messages.errMsg_INVALIDWEEKDAY;
			if( errCode == carter.ui.utils.ERRCODE_DELDEFFAILED)			return messages.errMsg_DELETEDEFFAILED;
			if( errCode == carter.ui.utils.ERRCODE_MISSINGPARAMS)			return messages.errMsg_MISSINGPARAMS;
			if( errCode == carter.ui.utils.ERRCODE_DEFISMODIFIEDBYOTHER)	return messages.errMsg_DEFISMODIFIEDBYOTHER;
			if( errCode == carter.ui.utils.ERRCODE_INTERNALSERVERR)			return messages.errMsg_INTERNALSERVERR;
			if( errCode == carter.ui.utils.ERRCODE_LAUNCHMODELSERVFAILED)	return messages.errMsg_LAUNCHMODELSERVFAILED;
			if( errCode == carter.ui.utils.ERRCODE_INVALIDPERMISSIONS)		return messages.errMsg_INVALIDPERMISSIONS;
			if( errCode == carter.ui.utils.ERRCODE_MODELSERVERNOTCONFIGURED)return messages.errMsg_MISSINGMODELSERVCONFIG;
			
			return null;
		},
		
		getDefStatusDescription: function(statCode) {
			if( statCode == carter.ui.utils.DEFSTAT_ACTIVE ) 				return messages.stat_Active;
			if( statCode == carter.ui.utils.DEFSTAT_INACTIVE) 				return messages.stat_Inactive;
			
			return null;
		},
		
		getActivityTypeDesc: function(actType){
			if( actType == "modelimport" ) {
				return messages.modelimport;
			}
			else if( actType == "modelremoved" ) {
				return messages.modelremoved;
			}
			else 
				return messages.unknown;
		}
    };

})();


